@@ -1,94 +1,48 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: XSLTProcessorApplet.java 469316 2006-10-30 23:32:59Z minchau $
  */
 package org.apache.xalan.client;
 
 import java.applet.Applet;
-
 import java.awt.Graphics;
-
-import java.net.URL;
-import java.net.MalformedURLException;
-
-import java.io.PrintWriter;
-import java.io.StringWriter;
-import java.io.StringReader;
 import java.io.IOException;
-import java.io.InputStream;
+import java.io.PrintWriter;
+import java.io.StringReader;
+import java.io.StringWriter;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.util.Hashtable;
+import java.util.Enumeration;
 
-import java.util.Properties;
-
-// Needed Xalan classes
-import org.apache.xalan.res.XSLMessages;
-import org.apache.xalan.res.XSLTErrorResources;
-
-// Needed TRaX classes
-import javax.xml.transform.Result;
-import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.TransformerConfigurationException;
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerException;
 import javax.xml.transform.Templates;
-import javax.xml.transform.Source;
-import javax.xml.transform.Result;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerConfigurationException;
+import javax.xml.transform.TransformerException;
+import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.stream.StreamSource;
 
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+
 /**
- * <meta name="usage" content="general"/>
  * Provides applet host for the XSLT processor. To perform transformations on an HTML client:
  * <ol>
  * <li>Use an &lt;applet&gt; tag to embed this applet in the HTML client.</li>
@@ -97,15 +51,23 @@ import javax.xml.transform.stream.StreamSource;
  * <li>Call the {@link #getHtmlText} method (or one of the transformToHtml() methods)
  * to perform the transformation and return the result as a String.</li>
  * </ol>
+ * 
+ * This class extends Applet which ultimately causes this class to implement Serializable.
+ * This is a serious restriction on this class. All fields that are not transient and not
+ * static are written-out/read-in during serialization. So even private fields essentially
+ * become part of the API. Developers need to take care when modifying fields.
+ * @xsl.usage general
  */
 public class XSLTProcessorApplet extends Applet
 {
 
   /**
    * The stylesheet processor.
-   * @serial
+   * This field is now transient because a 
+   * javax.xml.transform.TransformerFactory from JAXP 
+   * makes no claims to be serializable.
    */
-  TransformerFactory m_tfactory = null;
+  transient TransformerFactory m_tfactory = null;
 
   /**
    * @serial
@@ -250,6 +212,9 @@ public class XSLTProcessorApplet extends Applet
     // styleURL: Parameter description
     //----------------------------------------------------------------------
     param = getParameter(PARAM_styleURL);
+    
+    // stylesheet parameters
+    m_parameters = new Hashtable();
 
     if (param != null)
       setStyleURL(param);
@@ -402,15 +367,11 @@ public class XSLTProcessorApplet extends Applet
     m_attrValueToSet = value;
   }
 
-  /** 
-   * Stylesheet parameter key
-   */
-  transient String m_key;
 
   /** 
-   * Stylesheet parameter value
+   * Stylesheet parameter key/value pair stored in a hashtable
    */
-  transient String m_expression;
+  transient Hashtable m_parameters;  
 
   /**
    * Submit a stylesheet parameter.
@@ -421,8 +382,7 @@ public class XSLTProcessorApplet extends Applet
    */
   public void setStylesheetParam(String key, String expr)
   {
-    m_key = key;
-    m_expression = expr;
+    m_parameters.put(key, expr);
   }
 
   /**
@@ -575,7 +535,7 @@ public class XSLTProcessorApplet extends Applet
     catch (MalformedURLException e)
     {
       e.printStackTrace();
-      System.exit(-1);
+      throw new RuntimeException(e.getMessage());
     }      
     catch (Exception any_error)
     {
@@ -703,20 +663,24 @@ public class XSLTProcessorApplet extends Applet
 
         Transformer transformer = m_tfactory.newTransformer(xslSource);
 
-        if (null != m_key)
-          transformer.setParameter(m_key, m_expression);
-      
-         transformer.transform(xmlSource, result);
+        
+        Enumeration m_keys = m_parameters.keys();
+        while (m_keys.hasMoreElements()){
+          Object key = m_keys.nextElement();
+          Object expression = m_parameters.get(key);
+          transformer.setParameter((String) key, expression);
+        }
+        transformer.transform(xmlSource, result);
       }
       catch (TransformerConfigurationException tfe)
       {
         tfe.printStackTrace();
-        System.exit(-1);
+        throw new RuntimeException(tfe.getMessage());
       }
       catch (MalformedURLException e)
       {
         e.printStackTrace();
-        System.exit(-1);
+        throw new RuntimeException(e.getMessage());
       }
 	  
       this.showStatus("Transformation Done!");
@@ -752,7 +716,7 @@ public class XSLTProcessorApplet extends Applet
     {
       while (true)
       {
-        m_trustedWorker.yield();
+        Thread.yield();
 
         if (m_getData)  // Perform a transformation or get a document.
         {
@@ -785,7 +749,7 @@ public class XSLTProcessorApplet extends Applet
         {
           try
           {
-            m_trustedWorker.sleep(50);
+            Thread.sleep(50);
           }
           catch (InterruptedException ie)
           {
@@ -795,4 +759,21 @@ public class XSLTProcessorApplet extends Applet
       }
     }
   }
+  
+  // For compatiblity with old serialized objects
+  // We will change non-serialized fields and change methods
+  // and not have this break us.
+  private static final long serialVersionUID=4618876841979251422L;
+  
+  // For compatibility when de-serializing old objects
+  private void readObject(java.io.ObjectInputStream inStream) throws IOException, ClassNotFoundException 
+  {
+      inStream.defaultReadObject();
+      
+      // Needed assignment of non-serialized fields
+      
+      // A TransformerFactory is not guaranteed to be serializable, 
+      // so we create one here
+      m_tfactory = TransformerFactory.newInstance();
+  }      
 }
