@@ -1,80 +1,43 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: ExtensionHandlerJavaClass.java 469672 2006-10-31 21:56:19Z minchau $
  */
 
 package org.apache.xalan.extensions;
 
-import java.util.Vector;
-import java.lang.reflect.Method;
-import java.lang.reflect.Modifier;
+import java.io.IOException;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
-import java.io.IOException;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.util.Vector;
 
-import org.w3c.dom.Element;
-import org.w3c.dom.Node;
-import org.apache.xalan.transformer.TransformerImpl;
-import org.apache.xalan.templates.Stylesheet;
-import org.apache.xml.utils.QName;
-
-import org.apache.xpath.objects.XObject;
 import javax.xml.transform.TransformerException;
 
+import org.apache.xalan.templates.ElemTemplateElement;
+import org.apache.xalan.templates.Stylesheet;
+import org.apache.xalan.trace.ExtensionEvent;
+import org.apache.xalan.transformer.TransformerImpl;
+import org.apache.xpath.functions.FuncExtFunction;
+import org.apache.xpath.objects.XObject;
+
 /**
- * <meta name="usage" content="internal"/>
  * Represents an extension namespace for XPath that handles java classes.
  * It is recommended that the class URI be of the form:
  * <pre>
@@ -88,6 +51,7 @@ import javax.xml.transform.TransformerException;
  * element.
  *
  * @author <a href="mailto:garyp@firstech.com">Gary L Peskin</a>
+ * @xsl.usage internal
  */
 
 public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
@@ -168,8 +132,9 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
         Class[] paramTypes = methods[i].getParameterTypes();
         if ( (paramTypes.length == 2)
           && paramTypes[0].isAssignableFrom(
-                                     org.apache.xalan.extensions.XSLProcessorContext.class)
-          && paramTypes[1].isAssignableFrom(org.apache.xalan.templates.ElemExtensionCall.class) )
+                        org.apache.xalan.extensions.XSLProcessorContext.class)
+          && paramTypes[1].isAssignableFrom(
+                        org.apache.xalan.templates.ElemExtensionCall.class) )
         {
           return true;
         }
@@ -177,8 +142,7 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
     }
     return false;
   }
-
-
+  
   /**
    * Process a call to a function in the java class represented by
    * this <code>ExtensionHandlerJavaClass<code>.
@@ -229,23 +193,33 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
 
     try
     {
+      TransformerImpl trans = (exprContext != null) ?
+          (TransformerImpl)exprContext.getXPathContext().getOwnerObject() : null;
       if (funcName.equals("new")) {                   // Handle constructor call
 
         methodArgs = new Object[args.size()];
         convertedArgs = new Object[1][];
         for (int i = 0; i < methodArgs.length; i++)
         {
-          methodArgs[i] = args.elementAt(i);
+          methodArgs[i] = args.get(i);
         }
-        Constructor c = (Constructor) getFromCache(methodKey, null, methodArgs);
-        if (c != null)
+        Constructor c = null;
+        if (methodKey != null)
+          c = (Constructor) getFromCache(methodKey, null, methodArgs);
+        
+        if (c != null && !trans.getDebug())
         {
           try
           {
             paramTypes = c.getParameterTypes();
-            MethodResolver.convertParams(methodArgs, convertedArgs, paramTypes, exprContext);
+            MethodResolver.convertParams(methodArgs, convertedArgs, 
+                        paramTypes, exprContext);
             return c.newInstance(convertedArgs[0]);
           }
+          catch (InvocationTargetException ite)
+          {
+            throw ite;
+          }
           catch(Exception e)
           {
             // Must not have been the right one
@@ -255,8 +229,24 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
                                           methodArgs,
                                           convertedArgs,
                                           exprContext);
-        putToCache(methodKey, null, methodArgs, c);
-        return c.newInstance(convertedArgs[0]);
+        if (methodKey != null)
+          putToCache(methodKey, null, methodArgs, c);
+        
+        if (trans != null && trans.getDebug()) {            
+            trans.getTraceManager().fireExtensionEvent(new 
+                    ExtensionEvent(trans, c, convertedArgs[0]));
+            Object result;
+            try {            
+                result = c.newInstance(convertedArgs[0]);
+            } catch (Exception e) {
+                throw e;
+            } finally {
+                trans.getTraceManager().fireExtensionEndEvent(new 
+                        ExtensionEvent(trans, c, convertedArgs[0]));
+            }
+            return result;
+        } else
+            return c.newInstance(convertedArgs[0]);
       }
 
       else
@@ -268,15 +258,19 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
         convertedArgs = new Object[1][];
         for (int i = 0; i < methodArgs.length; i++)
         {
-          methodArgs[i] = args.elementAt(i);
+          methodArgs[i] = args.get(i);
         }
-        Method m = (Method) getFromCache(methodKey, null, methodArgs);
-        if (m != null)
+        Method m = null;
+        if (methodKey != null)
+          m = (Method) getFromCache(methodKey, null, methodArgs);
+        
+        if (m != null && !trans.getDebug())
         {
           try
           {
             paramTypes = m.getParameterTypes();
-            MethodResolver.convertParams(methodArgs, convertedArgs, paramTypes, exprContext);
+            MethodResolver.convertParams(methodArgs, convertedArgs, 
+                        paramTypes, exprContext);
             if (Modifier.isStatic(m.getModifiers()))
               return m.invoke(null, convertedArgs[0]);
             else
@@ -289,9 +283,20 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
               if (methodArgs.length <= nTargetArgs)
                 return m.invoke(m_defaultInstance, convertedArgs[0]);
               else  
-                return m.invoke(methodArgs[0], convertedArgs[0]);
+              {
+                targetObject = methodArgs[0];
+                
+                if (targetObject instanceof XObject)
+                  targetObject = ((XObject) targetObject).object();
+                  
+                return m.invoke(targetObject, convertedArgs[0]);
+              }
             }
           }
+          catch (InvocationTargetException ite)
+          {
+            throw ite;
+          }
           catch(Exception e)
           {
             // Must not have been the right one
@@ -322,21 +327,78 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
                                      convertedArgs,
                                      exprContext,
                                      resolveType);
-        putToCache(methodKey, null, methodArgs, m);
+        if (methodKey != null)
+          putToCache(methodKey, null, methodArgs, m);
 
-        if (MethodResolver.DYNAMIC == resolveType)          // First argument was object type
-          return m.invoke(targetObject, convertedArgs[0]);
+        if (MethodResolver.DYNAMIC == resolveType) {         // First argument was object type
+          if (trans != null && trans.getDebug()) {
+            trans.getTraceManager().fireExtensionEvent(m, targetObject, 
+                        convertedArgs[0]);
+            Object result;
+            try {
+                result = m.invoke(targetObject, convertedArgs[0]);
+            } catch (Exception e) {
+                throw e;
+            } finally {
+                trans.getTraceManager().fireExtensionEndEvent(m, targetObject, 
+                        convertedArgs[0]);
+            }
+            return result;
+          } else                  
+            return m.invoke(targetObject, convertedArgs[0]);
+        }
         else                                  // First arg was not object.  See if we need the implied object.
         {
-          if (Modifier.isStatic(m.getModifiers()))
-            return m.invoke(null, convertedArgs[0]);
+          if (Modifier.isStatic(m.getModifiers())) {
+            if (trans != null && trans.getDebug()) {
+              trans.getTraceManager().fireExtensionEvent(m, null, 
+                        convertedArgs[0]);
+              Object result;
+              try {
+                  result = m.invoke(null, convertedArgs[0]);
+              } catch (Exception e) {
+                throw e;
+              } finally {
+                trans.getTraceManager().fireExtensionEndEvent(m, null, 
+                        convertedArgs[0]);
+              }
+              return result;
+            } else                  
+              return m.invoke(null, convertedArgs[0]);
+          }
           else
           {
             if (null == m_defaultInstance)
             {
-              m_defaultInstance = m_classObj.newInstance();
+              if (trans != null && trans.getDebug()) {
+                trans.getTraceManager().fireExtensionEvent(new 
+                        ExtensionEvent(trans, m_classObj));
+                try {
+                    m_defaultInstance = m_classObj.newInstance();
+                } catch (Exception e) {
+                    throw e;
+                } finally {
+                    trans.getTraceManager().fireExtensionEndEvent(new 
+                        ExtensionEvent(trans, m_classObj));
+                }
+              }    else
+                  m_defaultInstance = m_classObj.newInstance();
             }
-            return m.invoke(m_defaultInstance, convertedArgs[0]);
+            if (trans != null && trans.getDebug()) {
+              trans.getTraceManager().fireExtensionEvent(m, m_defaultInstance, 
+                    convertedArgs[0]);
+              Object result;
+              try {
+                result = m.invoke(m_defaultInstance, convertedArgs[0]);
+              } catch (Exception e) {
+                throw e;
+              } finally {
+                trans.getTraceManager().fireExtensionEndEvent(m, 
+                        m_defaultInstance, convertedArgs[0]);
+              }
+              return result;
+            } else                  
+              return m.invoke(m_defaultInstance, convertedArgs[0]);
           }  
         }
 
@@ -344,11 +406,15 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
     }
     catch (InvocationTargetException ite)
     {
-      Throwable realException = ite.getTargetException();
-      if (realException instanceof Exception)
-        throw new TransformerException((Exception) realException);
-      else
-        throw new TransformerException(ite);
+      Throwable resultException = ite;
+      Throwable targetException = ite.getTargetException();
+ 
+      if (targetException instanceof TransformerException)
+        throw ((TransformerException)targetException);
+      else if (targetException != null)
+        resultException = targetException;
+            
+      throw new TransformerException(resultException);
     }
     catch (Exception e)
     {
@@ -357,6 +423,23 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
     }
   }
 
+  /**
+   * Process a call to an XPath extension function
+   *
+   * @param extFunction The XPath extension function
+   * @param args The arguments of the function call.
+   * @param exprContext The context in which this expression is being executed.
+   * @return the return value of the function evaluation.
+   * @throws TransformerException
+   */
+  public Object callFunction(FuncExtFunction extFunction,
+                             Vector args,
+                             ExpressionContext exprContext)
+      throws TransformerException
+  {
+    return callFunction(extFunction.getFunctionName(), args, 
+                        extFunction.getMethodKey(), exprContext);
+  }
 
   /**
    * Process a call to this extension namespace via an element. As a side
@@ -369,22 +452,15 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
    * @param element        The extension element being processed.
    * @param transformer      Handle to TransformerImpl.
    * @param stylesheetTree The compiled stylesheet tree.
-   * @param sourceTree     The root of the source tree (but don't assume
-   *                       it's a Document).
-   * @param sourceNode     The current context node.
-   * @param mode           The current mode.
    * @param methodKey      A key that uniquely identifies this element call.
    * @throws IOException           if loading trouble
    * @throws TransformerException          if parsing trouble
    */
 
   public void processElement(String localPart,
-                             Element element,
+                             ElemTemplateElement element,
                              TransformerImpl transformer,
                              Stylesheet stylesheetTree,
-                             Node sourceTree,
-                             Node sourceNode,
-                             QName mode,
                              Object methodKey)
     throws TransformerException, IOException
   {
@@ -396,8 +472,22 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
       try
       {
         m = MethodResolver.getElementMethod(m_classObj, localPart);
-        if ( (null == m_defaultInstance) && !Modifier.isStatic(m.getModifiers()) )
-          m_defaultInstance = m_classObj.newInstance();
+        if ( (null == m_defaultInstance) && 
+                !Modifier.isStatic(m.getModifiers()) ) {
+          if (transformer.getDebug()) {            
+            transformer.getTraceManager().fireExtensionEvent(
+                    new ExtensionEvent(transformer, m_classObj));
+            try {
+              m_defaultInstance = m_classObj.newInstance();
+            } catch (Exception e) {
+              throw e;
+            } finally {
+              transformer.getTraceManager().fireExtensionEndEvent(
+                    new ExtensionEvent(transformer, m_classObj));
+            }
+          } else 
+            m_defaultInstance = m_classObj.newInstance();
+        }
       }
       catch (Exception e)
       {
@@ -408,14 +498,35 @@ public class ExtensionHandlerJavaClass extends ExtensionHandlerJava
     }
 
     XSLProcessorContext xpc = new XSLProcessorContext(transformer, 
-                                                      stylesheetTree,
-                                                      sourceTree, 
-                                                      sourceNode, 
-                                                      mode);
+                                                      stylesheetTree);
 
     try
     {
-      result = m.invoke(m_defaultInstance, new Object[] {xpc, element});
+      if (transformer.getDebug()) {
+        transformer.getTraceManager().fireExtensionEvent(m, m_defaultInstance, 
+                new Object[] {xpc, element});
+        try {
+          result = m.invoke(m_defaultInstance, new Object[] {xpc, element});
+        } catch (Exception e) {
+          throw e;
+        } finally {
+          transformer.getTraceManager().fireExtensionEndEvent(m, 
+                m_defaultInstance, new Object[] {xpc, element});
+        }
+      } else                  
+        result = m.invoke(m_defaultInstance, new Object[] {xpc, element});
+    }
+    catch (InvocationTargetException e)
+    {
+      Throwable targetException = e.getTargetException();
+      
+      if (targetException instanceof TransformerException)
+        throw (TransformerException)targetException;
+      else if (targetException != null)
+        throw new TransformerException (targetException.getMessage (), 
+                targetException);
+      else
+        throw new TransformerException (e.getMessage (), e);
     }
     catch (Exception e)
     {
