@@ -1,87 +1,51 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: ExtensionHandlerJavaPackage.java 469672 2006-10-31 21:56:19Z minchau $
  */
 package org.apache.xalan.extensions;
 
-import java.util.Hashtable;
-import java.util.Vector;
-import java.lang.reflect.Method;
-import java.lang.reflect.Modifier;
+import java.io.IOException;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
-import java.io.IOException;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.util.Vector;
 
-import org.w3c.dom.Element;
-import org.w3c.dom.Node;
-import org.apache.xalan.transformer.TransformerImpl;
-import org.apache.xalan.templates.Stylesheet;
-import org.apache.xml.utils.QName;
-
-import org.apache.xpath.objects.XObject;
 import javax.xml.transform.TransformerException;
 
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+import org.apache.xalan.templates.ElemTemplateElement;
+import org.apache.xalan.templates.Stylesheet;
+import org.apache.xalan.trace.ExtensionEvent;
+import org.apache.xalan.transformer.TransformerImpl;
+import org.apache.xpath.functions.FuncExtFunction;
+import org.apache.xpath.objects.XObject;
+
 /**
- * <meta name="usage" content="internal"/>
  * Represents an extension namespace for XPath that handles java packages
  * that may be fully or partially specified.
  * It is recommended that the class URI be of one of the following forms:
  * <pre>
  *   xalan://partial.class.name
  *   xalan://
- *   http://xml.apache.org/xslt/java (which is the same as xalan://)
+ *   http://xml.apache.org/xalan/java (which is the same as xalan://)
  * </pre>
  * However, we do not enforce this.  If the class name contains a
  * a /, we only use the part to the right of the rightmost slash.
@@ -92,6 +56,7 @@ import javax.xml.transform.TransformerException;
  *
  * @author <a href="mailto:garyp@firstech.com">Gary L Peskin</a>
  *
+ * @xsl.usage internal
  */
 
 
@@ -156,7 +121,7 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
    * Tests whether a certain element name is known within this namespace.
    * Looks for a method with the appropriate name and signature.
    * This method examines both static and instance methods.
-   * @param function name of the function being tested
+   * @param element name of the element being tested
    * @return true if its known, false if not.
    */
 
@@ -228,7 +193,8 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
    *
    * @param funcName Function name.
    * @param args     The arguments of the function call.
-   *
+   * @param methodKey A key that uniquely identifies this class and method call.
+   * @param exprContext The context in which this expression is being executed.
    * @return the return value of the function evaluation.
    *
    * @throws TransformerException          if parsing trouble
@@ -252,16 +218,20 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
 
     try
     {
-
+      TransformerImpl trans = (exprContext != null) ?
+        (TransformerImpl)exprContext.getXPathContext().getOwnerObject() : null;
       if (funcName.endsWith(".new")) {                   // Handle constructor call
 
         methodArgs = new Object[args.size()];
         convertedArgs = new Object[1][];
         for (int i = 0; i < methodArgs.length; i++)
         {
-          methodArgs[i] = args.elementAt(i);
+          methodArgs[i] = args.get(i);
         }
-        Constructor c = (Constructor) getFromCache(methodKey, null, methodArgs);
+        
+        Constructor c = (methodKey != null) ?
+          (Constructor) getFromCache(methodKey, null, methodArgs) : null;
+        
         if (c != null)
         {
           try
@@ -270,6 +240,10 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
             MethodResolver.convertParams(methodArgs, convertedArgs, paramTypes, exprContext);
             return c.newInstance(convertedArgs[0]);
           }
+          catch (InvocationTargetException ite)
+          {
+            throw ite;
+          }
           catch(Exception e)
           {
             // Must not have been the right one
@@ -288,8 +262,22 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
                                           methodArgs,
                                           convertedArgs,
                                           exprContext);
-        putToCache(methodKey, null, methodArgs, c);
-        return c.newInstance(convertedArgs[0]);
+        if (methodKey != null)
+          putToCache(methodKey, null, methodArgs, c);
+        
+        if (trans != null && trans.getDebug()) {
+            trans.getTraceManager().fireExtensionEvent(new ExtensionEvent(trans, c, convertedArgs[0]));            
+            Object result;
+            try {
+                result = c.newInstance(convertedArgs[0]);
+            } catch (Exception e) {
+                throw e;
+            } finally {
+                trans.getTraceManager().fireExtensionEndEvent(new ExtensionEvent(trans, c, convertedArgs[0]));
+            }
+            return result;
+        } else
+            return c.newInstance(convertedArgs[0]);
       }
 
       else if (-1 != lastDot) {                         // Handle static method call
@@ -298,10 +286,12 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
         convertedArgs = new Object[1][];
         for (int i = 0; i < methodArgs.length; i++)
         {
-          methodArgs[i] = args.elementAt(i);
+          methodArgs[i] = args.get(i);
         }
-        Method m = (Method) getFromCache(methodKey, null, methodArgs);
-        if (m != null)
+        Method m = (methodKey != null) ?
+          (Method) getFromCache(methodKey, null, methodArgs) : null;
+        
+        if (m != null && !trans.getDebug())
         {
           try
           {
@@ -309,6 +299,10 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
             MethodResolver.convertParams(methodArgs, convertedArgs, paramTypes, exprContext);
             return m.invoke(null, convertedArgs[0]);
           }
+          catch (InvocationTargetException ite)
+          {
+            throw ite;
+          }
           catch(Exception e)
           {
             // Must not have been the right one
@@ -330,27 +324,44 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
                                      convertedArgs,
                                      exprContext,
                                      MethodResolver.STATIC_ONLY);
-        putToCache(methodKey, null, methodArgs, m);
-        return m.invoke(null, convertedArgs[0]);
+        if (methodKey != null)
+          putToCache(methodKey, null, methodArgs, m);
+        
+        if (trans != null && trans.getDebug()) {
+            trans.getTraceManager().fireExtensionEvent(m, null, convertedArgs[0]);            
+            Object result;
+            try {
+                result = m.invoke(null, convertedArgs[0]);
+            } catch (Exception e) {
+                throw e;
+            } finally {
+                trans.getTraceManager().fireExtensionEndEvent(m, null, convertedArgs[0]);
+            }
+            return result;
+        }
+        else
+            return m.invoke(null, convertedArgs[0]);
       }
 
       else {                                            // Handle instance method call
 
         if (args.size() < 1)
         {
-          throw new TransformerException("Instance method call to method " + funcName
-                                    + " requires an Object instance as first argument");
+          throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_INSTANCE_MTHD_CALL_REQUIRES, new Object[]{funcName })); //"Instance method call to method " + funcName
+                                    //+ " requires an Object instance as first argument");
         }
-        targetObject = args.elementAt(0);
+        targetObject = args.get(0);
         if (targetObject instanceof XObject)          // Next level down for XObjects
           targetObject = ((XObject) targetObject).object();
         methodArgs = new Object[args.size() - 1];
         convertedArgs = new Object[1][];
         for (int i = 0; i < methodArgs.length; i++)
         {
-          methodArgs[i] = args.elementAt(i+1);
+          methodArgs[i] = args.get(i+1);
         }
-        Method m = (Method) getFromCache(methodKey, targetObject, methodArgs);
+        Method m = (methodKey != null) ?
+          (Method) getFromCache(methodKey, targetObject, methodArgs) : null;
+        
         if (m != null)
         {
           try
@@ -359,6 +370,10 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
             MethodResolver.convertParams(methodArgs, convertedArgs, paramTypes, exprContext);
             return m.invoke(targetObject, convertedArgs[0]);
           }
+          catch (InvocationTargetException ite)
+          {
+            throw ite;
+          }
           catch(Exception e)
           {
             // Must not have been the right one
@@ -371,17 +386,35 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
                                      convertedArgs,
                                      exprContext,
                                      MethodResolver.INSTANCE_ONLY);
-        putToCache(methodKey, targetObject, methodArgs, m);
-        return m.invoke(targetObject, convertedArgs[0]);
+        if (methodKey != null)
+          putToCache(methodKey, targetObject, methodArgs, m);
+        
+        if (trans != null && trans.getDebug()) {
+            trans.getTraceManager().fireExtensionEvent(m, targetObject, convertedArgs[0]);            
+            Object result;
+            try {
+                result = m.invoke(targetObject, convertedArgs[0]);
+            } catch (Exception e) {
+                throw e;
+            } finally {
+                trans.getTraceManager().fireExtensionEndEvent(m, targetObject, convertedArgs[0]);
+            }
+            return result;
+        } else       
+            return m.invoke(targetObject, convertedArgs[0]);
       }
     }
     catch (InvocationTargetException ite)
     {
-      Throwable realException = ite.getTargetException();
-      if (realException instanceof Exception)
-        throw new TransformerException((Exception) realException);
-      else
-        throw new TransformerException(ite);
+      Throwable resultException = ite;
+      Throwable targetException = ite.getTargetException();
+ 
+      if (targetException instanceof TransformerException)
+        throw ((TransformerException)targetException);
+      else if (targetException != null)
+        resultException = targetException;
+            
+      throw new TransformerException(resultException);
     }
     catch (Exception e)
     {
@@ -390,6 +423,23 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
     }
   }
 
+  /**
+   * Process a call to an XPath extension function
+   *
+   * @param extFunction The XPath extension function
+   * @param args The arguments of the function call.
+   * @param exprContext The context in which this expression is being executed.
+   * @return the return value of the function evaluation.
+   * @throws TransformerException
+   */
+  public Object callFunction(FuncExtFunction extFunction,
+                             Vector args,
+                             ExpressionContext exprContext)
+      throws TransformerException
+  {
+    return callFunction(extFunction.getFunctionName(), args, 
+                        extFunction.getMethodKey(), exprContext);
+  }
 
   /**
    * Process a call to this extension namespace via an element. As a side
@@ -400,23 +450,15 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
    * @param element        The extension element being processed.
    * @param transformer      Handle to TransformerImpl.
    * @param stylesheetTree The compiled stylesheet tree.
-   * @param mode           The current mode.
-   * @param sourceTree     The root of the source tree (but don't assume
-   *                       it's a Document).
-   * @param sourceNode     The current context node.
-   * @param mode           The current mode.
    * @param methodKey      A key that uniquely identifies this element call.
    * @throws IOException           if loading trouble
    * @throws TransformerException          if parsing trouble
    */
 
   public void processElement (String localPart,
-                              Element element,
+                              ElemTemplateElement element,
                               TransformerImpl transformer,
                               Stylesheet stylesheetTree,
-                              Node sourceTree,
-                              Node sourceNode,
-                              QName mode,
                               Object methodKey)
     throws TransformerException, IOException
   {
@@ -431,7 +473,7 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
         String fullName = m_className + localPart;
         int lastDot = fullName.lastIndexOf(".");
         if (lastDot < 0)
-          throw new TransformerException("Invalid element name specified " + fullName);
+          throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_INVALID_ELEMENT_NAME, new Object[]{fullName })); //"Invalid element name specified " + fullName);
         try
         {
           classObj = getClassForName(fullName.substring(0, lastDot));
@@ -443,7 +485,7 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
         localPart = fullName.substring(lastDot + 1);
         m = MethodResolver.getElementMethod(classObj, localPart);
         if (!Modifier.isStatic(m.getModifiers()))
-          throw new TransformerException("Element name method must be static " + fullName);
+          throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_ELEMENT_NAME_METHOD_STATIC, new Object[]{fullName })); //"Element name method must be static " + fullName);
       }
       catch (Exception e)
       {
@@ -454,14 +496,33 @@ public class ExtensionHandlerJavaPackage extends ExtensionHandlerJava
     }
 
     XSLProcessorContext xpc = new XSLProcessorContext(transformer, 
-                                                      stylesheetTree,
-                                                      sourceTree, 
-                                                      sourceNode, 
-                                                      mode);
+                                                      stylesheetTree);
 
     try
     {
-      result = m.invoke(null, new Object[] {xpc, element});
+      if (transformer.getDebug()) {
+          transformer.getTraceManager().fireExtensionEvent(m, null, new Object[] {xpc, element});
+        try {
+            result = m.invoke(null, new Object[] {xpc, element});
+        } catch (Exception e) {
+            throw e;
+        } finally {            
+            transformer.getTraceManager().fireExtensionEndEvent(m, null, new Object[] {xpc, element});
+        }
+      } else
+        result = m.invoke(null, new Object[] {xpc, element});
+    }
+    catch (InvocationTargetException ite)
+    {
+      Throwable resultException = ite;
+      Throwable targetException = ite.getTargetException();
+ 
+      if (targetException instanceof TransformerException)
+        throw ((TransformerException)targetException);
+      else if (targetException != null)
+        resultException = targetException;
+            
+      throw new TransformerException(resultException);
     }
     catch (Exception e)
     {
