@@ -1,76 +1,43 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: MethodResolver.java 468637 2006-10-28 06:51:02Z minchau $
  */
-
 package org.apache.xalan.extensions;
 
-import java.lang.reflect.Method;
 import java.lang.reflect.Constructor;
+import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 
+import javax.xml.transform.TransformerException;
+
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+import org.apache.xml.dtm.DTM;
+import org.apache.xml.dtm.DTMIterator;
+import org.apache.xml.dtm.ref.DTMNodeIterator;
+import org.apache.xpath.objects.XObject;
+import org.apache.xpath.objects.XRTreeFrag;
+import org.apache.xpath.objects.XString;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.w3c.dom.traversal.NodeIterator;
 
-import org.apache.xpath.objects.XObject;
-import org.apache.xpath.objects.XString;
-import org.apache.xpath.objects.XRTreeFrag;
-
-import javax.xml.transform.TransformerException;
-
 /**
  * Utility class to help resolve method overloading with Xalan XSLT 
  * argument types.
@@ -182,7 +149,7 @@ public class MethodResolver
     }
     /*** This is commented out until we can do a better object -> object scoring 
     else if (bestScoreCount > 1)
-      throw new TransformerException("More than one best match for constructor for "
+      throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_MORE_MATCH_CONSTRUCTOR, new Object[]{classObj.getName()})); //"More than one best match for constructor for "
                                                                    + classObj.getName());
     ***/
     else
@@ -217,6 +184,8 @@ public class MethodResolver
   {
     // System.out.println("---> Looking for method: "+name);
     // System.out.println("---> classObj: "+classObj);
+    if (name.indexOf("-")>0)
+      name = replaceDash(name);
     Method bestMethod = null;
     Class[] bestParamTypes = null;
     Method[] methods = classObj.getMethods();
@@ -309,14 +278,34 @@ public class MethodResolver
     }
     /*** This is commented out until we can do a better object -> object scoring 
     else if (bestScoreCount > 1)
-      throw new TransformerException("More than one best match for method " + name);
+      throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_MORE_MATCH_METHOD, new Object[]{name})); //"More than one best match for method " + name);
     ***/
     else
       convertParams(argsIn, argsOut, bestParamTypes, exprContext);
     
     return bestMethod;
   }
-
+  
+  /**
+   * To support EXSLT extensions, convert names with dash to allowable Java names: 
+   * e.g., convert abc-xyz to abcXyz.
+   * Note: dashes only appear in middle of an EXSLT function or element name.
+   */
+  private static String replaceDash(String name)
+  {
+    char dash = '-';
+    StringBuffer buff = new StringBuffer("");
+    for (int i=0; i<name.length(); i++)
+    {
+      if (name.charAt(i) == dash)
+      {}
+      else if (i > 0 && name.charAt(i-1) == dash)
+        buff.append(Character.toUpperCase(name.charAt(i)));
+      else
+        buff.append(name.charAt(i));
+    }
+    return buff.toString();
+  }
   
   /**
    * Given the name of a method, figure out the resolution of 
@@ -338,7 +327,6 @@ public class MethodResolver
     Method bestMethod = null;
     Method[] methods = classObj.getMethods();
     int nMethods = methods.length;
-    int bestScore = Integer.MAX_VALUE;
     int bestScoreCount = 0;
     for(int i = 0; i < nMethods; i++)
     {
@@ -348,33 +336,13 @@ public class MethodResolver
       {
         Class[] paramTypes = method.getParameterTypes();
         if ( (paramTypes.length == 2)
-           && paramTypes[1].isAssignableFrom(org.apache.xalan.templates.ElemExtensionCall.class) )
+           && paramTypes[1].isAssignableFrom(org.apache.xalan.templates.ElemExtensionCall.class)
+                                         && paramTypes[0].isAssignableFrom(org.apache.xalan.extensions.XSLProcessorContext.class) )
         {
-          int score = -1;
-          if (paramTypes[0].isAssignableFrom(
-                                      org.apache.xalan.extensions.XSLProcessorContext.class))
-          {
-            score = 10;
-          }
-          /*******
-          else if (paramTypes[0].isAssignableFrom(
-                                      org.apace.xalan.xslt.XSLProcessorContext.class))
-          {
-            score = 5;
-          }
-          ********/
-          else 
-            continue;
-
-          if (score < bestScore)
-          {
-            // System.out.println("Assigning best method: "+method);
+          if ( ++bestScoreCount == 1 )
             bestMethod = method;
-            bestScore = score;
-            bestScoreCount = 1;
-          }
-          else if (score == bestScore)
-            bestScoreCount++;
+          else
+            break;
         }
       }
     }
@@ -385,7 +353,7 @@ public class MethodResolver
                                                                         name, 0, null));
     }
     else if (bestScoreCount > 1)
-      throw new TransformerException("More than one best match for element method " + name);
+      throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_MORE_MATCH_ELEMENT, new Object[]{name})); //"More than one best match for element method " + name);
     
     return bestMethod;
   }
@@ -455,7 +423,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_UNKNOWN
    * (i.e. some unknown Java object) to allowed Java types.
    */
-  static ConversionInfo[] m_javaObjConversions = {
+  private final static ConversionInfo[] m_javaObjConversions = {
     new ConversionInfo(Double.TYPE, 11),
     new ConversionInfo(Float.TYPE, 12),
     new ConversionInfo(Long.TYPE, 13),
@@ -470,7 +438,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_BOOLEAN
    * to allowed Java types.
    */
-  static ConversionInfo[] m_booleanConversions = {
+  private final static ConversionInfo[] m_booleanConversions = {
     new ConversionInfo(Boolean.TYPE, 0),
     new ConversionInfo(java.lang.Boolean.class, 1),
     new ConversionInfo(java.lang.Object.class, 2),
@@ -481,7 +449,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_NUMBER
    * to allowed Java types.
    */
-  static ConversionInfo[] m_numberConversions = {
+  private final static ConversionInfo[] m_numberConversions = {
     new ConversionInfo(Double.TYPE, 0),
     new ConversionInfo(java.lang.Double.class, 1),
     new ConversionInfo(Float.TYPE, 3),
@@ -499,7 +467,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_STRING
    * to allowed Java types.
    */
-  static ConversionInfo[] m_stringConversions = {
+  private final static ConversionInfo[] m_stringConversions = {
     new ConversionInfo(java.lang.String.class, 0),
     new ConversionInfo(java.lang.Object.class, 1),
     new ConversionInfo(Character.TYPE, 2),
@@ -516,7 +484,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_RTREEFRAG
    * to allowed Java types.
    */
-  static ConversionInfo[] m_rtfConversions = {
+  private final static ConversionInfo[] m_rtfConversions = {
     new ConversionInfo(org.w3c.dom.traversal.NodeIterator.class, 0),
     new ConversionInfo(org.w3c.dom.NodeList.class, 1),
     new ConversionInfo(org.w3c.dom.Node.class, 2),
@@ -536,7 +504,7 @@ public class MethodResolver
    * Specification of conversions from XSLT type CLASS_NODESET
    * to allowed Java types.  (This is the same as for CLASS_RTREEFRAG)
    */
-  static ConversionInfo[] m_nodesetConversions = {
+  private final static ConversionInfo[] m_nodesetConversions = {
     new ConversionInfo(org.w3c.dom.traversal.NodeIterator.class, 0),
     new ConversionInfo(org.w3c.dom.NodeList.class, 1),
     new ConversionInfo(org.w3c.dom.Node.class, 2),
@@ -556,7 +524,7 @@ public class MethodResolver
    * Order is significant in the list below, based on 
    * XObject.CLASS_XXX values.
    */
-  static ConversionInfo[][] m_conversions = 
+  private final static ConversionInfo[][] m_conversions = 
   {
     m_javaObjConversions, // CLASS_UNKNOWN = 0;
     m_booleanConversions, // CLASS_BOOLEAN = 1;
@@ -630,15 +598,53 @@ public class MethodResolver
       {
         // If we get here, we haven't made a match on this parameter using 
         // the ConversionInfo array.  We now try to handle the object -> object
-        // mapping which we can't handle through the array mechanism.
+        // mapping which we can't handle through the array mechanism.  To do this,
+        // we must determine the class of the argument passed from the stylesheet.
+
+        // If we were passed a subclass of XObject, representing one of the actual
+        // XSLT types, and we are here, we reject this extension method as a candidate
+        // because a match should have been made using the ConversionInfo array.  If we 
+        // were passed an XObject that encapsulates a non-XSLT type or we
+        // were passed a non-XSLT type directly, we continue.
+
+        // The current implementation (contributed by Kelly Campbell <camk@channelpoint.com>)
+        // checks to see if we were passed an XObject from the XSLT stylesheet.  If not,
+        // we use the class of the object that was passed and make sure that it will
+        // map to the class type of the parameter in the extension function.
+        // If we were passed an XObject, we attempt to get the class of the actual
+        // object encapsulated inside the XObject.  If the encapsulated object is null,
+        // we judge this method as a match but give it a low score.  
+        // If the encapsulated object is not null, we use its type to determine
+        // whether this java method is a valid match for this extension function call.
+        // This approach eliminates the NullPointerException in the earlier implementation
+        // that resulted from passing an XObject encapsulating the null java object.
+                                
         // TODO:  This needs to be improved to assign relative scores to subclasses,
-        // etc.  I plan to do this very soon.  Gary Peskin 29-Oct-2000.
+        // etc. 
 
         if (XObject.CLASS_UNKNOWN == xsltClassType)
         {
-          Class realClass = (xsltObj instanceof XObject)
-                              ? ((XObject) xsltObj).object().getClass()
-                              : xsltObj.getClass();
+          Class realClass = null;
+
+          if (xsltObj instanceof XObject)
+          {
+            Object realObj = ((XObject) xsltObj).object();
+            if (null != realObj)
+            {
+              realClass = realObj.getClass();
+            }
+            else
+            {
+              // do the same as if we were passed XObject.CLASS_NULL
+              score += 10;
+              continue;
+            }
+          }
+          else
+          {
+            realClass = xsltObj.getClass();
+          }
+
           if (javaClass.isAssignableFrom(realClass))
           {
             score += 0;         // TODO: To be assigned based on subclass "distance"
@@ -730,7 +736,8 @@ public class MethodResolver
           if ( (javaClass == NodeIterator.class) ||
                (javaClass == java.lang.Object.class) )
           {
-            return ((XRTreeFrag) xobj).asNodeIterator();
+            DTMIterator dtmIter = ((XRTreeFrag) xobj).asNodeIterator();
+            return new DTMNodeIterator(dtmIter);
           }
           else if (javaClass == NodeList.class)
           {
@@ -740,7 +747,10 @@ public class MethodResolver
           // else if(Node.class.isAssignableFrom(javaClass))
           else if(javaClass == Node.class)
           {
-            return xobj.rtree().getFirstChild();
+            DTMIterator iter = ((XRTreeFrag) xobj).asNodeIterator();
+            int rootHandle = iter.nextNode();
+            DTM dtm = iter.getDTM(rootHandle);
+            return dtm.getNode(dtm.getFirstChild(rootHandle));
           }
           else if(javaClass == java.lang.String.class)
           {
@@ -756,7 +766,11 @@ public class MethodResolver
           }
           else
           {
-            Node child = xobj.rtree().getFirstChild();
+            DTMIterator iter = ((XRTreeFrag) xobj).asNodeIterator();
+            int rootHandle = iter.nextNode();
+            DTM dtm = iter.getDTM(rootHandle);
+            Node child = dtm.getNode(dtm.getFirstChild(rootHandle));
+
             if(javaClass.isAssignableFrom(child.getClass()))
               return child;
             else
@@ -782,16 +796,20 @@ public class MethodResolver
           // else if(NodeList.class.isAssignableFrom(javaClass))
           else if(javaClass == NodeList.class)
           {
-            return xobj.mutableNodeset();
+            return xobj.nodelist();
           }
           // Same comment as above
           // else if(Node.class.isAssignableFrom(javaClass))
           else if(javaClass == Node.class)
           {
-            // Xalan ensures that nodeset() always returns an
+            // Xalan ensures that iter() always returns an
             // iterator positioned at the beginning.
-            NodeIterator ni = xobj.nodeset();
-            return ni.nextNode(); // may be null.
+            DTMIterator ni = xobj.iter();
+            int handle = ni.nextNode();
+            if (handle != DTM.NULL)
+              return ni.getDTM(handle).getNode(handle); // may be null.
+            else
+              return null;
           }
           else if(javaClass == java.lang.String.class)
           {
@@ -807,7 +825,10 @@ public class MethodResolver
           }
           else
           {
-            Node child = xobj.nodeset().nextNode();
+            DTMIterator iter = xobj.iter();
+            int childHandle = iter.nextNode();
+            DTM dtm = iter.getDTM(childHandle);
+            Node child = dtm.getNode(childHandle);
             if(javaClass.isAssignableFrom(child.getClass()))
               return child;
             else
@@ -822,22 +843,30 @@ public class MethodResolver
       
     } // end if if(xsltObj instanceof XObject)
     
-    // At this point, we have a raw java object.
-    if(javaClass == java.lang.String.class)
+    // At this point, we have a raw java object, not an XObject.
+    if (null != xsltObj)
     {
-      return xsltObj.toString();
-    }
-    else if(javaClass.isPrimitive())
-    {
-      // Assume a number conversion
-      XString xstr = new XString(xsltObj.toString());
-      double num = xstr.num();
-      return convertDoubleToNumber(num, javaClass);
-    }
-    else if(javaClass == java.lang.Class.class)
-    {
-      return xsltObj.getClass();
-    }
+      if(javaClass == java.lang.String.class)
+      {
+        return xsltObj.toString();
+      }
+      else if(javaClass.isPrimitive())
+      {
+        // Assume a number conversion
+        XString xstr = new XString(xsltObj.toString());
+        double num = xstr.num();
+        return convertDoubleToNumber(num, javaClass);
+      }
+      else if(javaClass == java.lang.Class.class)
+      {
+        return xsltObj.getClass();
+      }
+      else
+      {
+        // Just pass the object directly, and hope for the best.
+        return xsltObj;
+      }
+                }
     else
     {
       // Just pass the object directly, and hope for the best.
