@@ -1,116 +1,93 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: StylesheetHandler.java 468640 2006-10-28 06:53:53Z minchau $
  */
 package org.apache.xalan.processor;
 
-import java.net.URL;
-
-import java.io.IOException;
-
-import javax.xml.transform.sax.TemplatesHandler;
-import javax.xml.transform.Templates;
-import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerConfigurationException;
-
 import java.util.Stack;
 
+import javax.xml.transform.ErrorListener;
+import javax.xml.transform.Source;
+import javax.xml.transform.SourceLocator;
+import javax.xml.transform.Templates;
+import javax.xml.transform.TransformerConfigurationException;
+import javax.xml.transform.TransformerException;
+import javax.xml.transform.sax.TemplatesHandler;
+
+import org.apache.xalan.extensions.ExpressionVisitor;
 import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
 import org.apache.xalan.templates.Constants;
+import org.apache.xalan.templates.ElemForEach;
 import org.apache.xalan.templates.ElemTemplateElement;
-import org.apache.xalan.templates.ElemUnknown;
-import org.apache.xalan.templates.StylesheetRoot;
 import org.apache.xalan.templates.Stylesheet;
+import org.apache.xalan.templates.StylesheetRoot;
+import org.apache.xml.utils.BoolStack;
+import org.apache.xml.utils.NamespaceSupport2;
 import org.apache.xml.utils.NodeConsumer;
 import org.apache.xml.utils.PrefixResolver;
+import org.apache.xml.utils.SAXSourceLocator;
 import org.apache.xml.utils.XMLCharacterRecognizer;
-import org.apache.xml.utils.BoolStack;
-import org.apache.xpath.compiler.FunctionTable;
-import org.apache.xpath.compiler.XPathParser;
-import org.apache.xpath.functions.Function;
-import org.apache.xpath.XPathFactory;
 import org.apache.xpath.XPath;
+import org.apache.xpath.compiler.FunctionTable;
+import org.apache.xpath.functions.Function;
 
 import org.w3c.dom.Node;
 
 import org.xml.sax.Attributes;
-import org.xml.sax.ContentHandler;
-import org.xml.sax.DTDHandler;
-import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
 import org.xml.sax.Locator;
-import org.xml.sax.helpers.NamespaceSupport;
 import org.xml.sax.helpers.DefaultHandler;
-
-import javax.xml.transform.SourceLocator;
-import javax.xml.transform.ErrorListener;
-import javax.xml.transform.TransformerException;
-
-import org.apache.xml.utils.SAXSourceLocator;
+import org.xml.sax.helpers.NamespaceSupport;
 
 /**
- * <meta name="usage" content="advanced"/>
  * Initializes and processes a stylesheet via SAX events.
  * This class acts as essentially a state machine, maintaining
  * a ContentHandler stack, and pushing appropriate content
  * handlers as parse events occur.
+ * @xsl.usage advanced
  */
 public class StylesheetHandler extends DefaultHandler
         implements TemplatesHandler, PrefixResolver, NodeConsumer
 {
 
+
+  /**
+   * The function table of XPath and XSLT;
+   */
+  private FunctionTable m_funcTable = new FunctionTable();
+  
+  /**
+   * The flag for the setting of the optimize feature;
+   */
+  private boolean m_optimize = true;
+  
+  /**
+   * The flag for the setting of the incremental feature;
+   */
+  private boolean m_incremental = false;
+  
+  /**
+   * The flag for the setting of the source_location feature;
+   */
+  private boolean m_source_location = false;
+  
   /**
    * Create a StylesheetHandler object, creating a root stylesheet
    * as the target.
@@ -123,17 +100,26 @@ public class StylesheetHandler extends DefaultHandler
   public StylesheetHandler(TransformerFactoryImpl processor)
           throws TransformerConfigurationException
   {
+    Class func = org.apache.xalan.templates.FuncDocument.class;
+    m_funcTable.installFunction("document", func);
 
+    // func = new org.apache.xalan.templates.FuncKey();
+    // FunctionTable.installFunction("key", func);
+    func = org.apache.xalan.templates.FuncFormatNumb.class;
+
+    m_funcTable.installFunction("format-number", func);
+
+    m_optimize =((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_OPTIMIZE)).booleanValue();
+    m_incremental = ((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_INCREMENTAL)).booleanValue();
+    m_source_location = ((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_SOURCE_LOCATION)).booleanValue();
     // m_schema = new XSLTSchema();
     init(processor);
+    
   }
 
-  /**
-   * Static flag to let us know if the XPath functions table
-   * has been initialized.
-   */
-  private static boolean m_xpathFunctionsInited = false;
-
   /**
    * Do common initialization.
    *
@@ -141,30 +127,6 @@ public class StylesheetHandler extends DefaultHandler
    */
   void init(TransformerFactoryImpl processor)
   {
-
-    // Not sure about double-check of this flag, but
-    // it seems safe...
-    if (false == m_xpathFunctionsInited)
-    {
-      synchronized (this)
-      {
-        if (false == m_xpathFunctionsInited)
-        {
-          m_xpathFunctionsInited = true;
-
-          Function func = new org.apache.xalan.templates.FuncDocument();
-
-          FunctionTable.installFunction("document", func);
-
-          // func = new org.apache.xalan.templates.FuncKey();
-          // FunctionTable.installFunction("key", func);
-          func = new org.apache.xalan.templates.FuncFormatNumb();
-
-          FunctionTable.installFunction("format-number", func);
-        }
-      }
-    }
-
     m_stylesheetProcessor = processor;
 
     // Set the initial content handler.
@@ -186,11 +148,15 @@ public class StylesheetHandler extends DefaultHandler
    * @throws javax.xml.transform.TransformerException if the expression can not be processed.
    * @see <a href="http://www.w3.org/TR/xslt#section-Expressions">Section 4 Expressions in XSLT Specification</a>
    */
-  public XPath createXPath(String str)
+  public XPath createXPath(String str, ElemTemplateElement owningTemplate)
           throws javax.xml.transform.TransformerException
   {
     ErrorListener handler = m_stylesheetProcessor.getErrorListener();
-    return new XPath(str, getLocator(), this, XPath.SELECT, handler);
+    XPath xpath = new XPath(str, owningTemplate, this, XPath.SELECT, handler, 
+            m_funcTable);
+    // Visit the expression, registering namespaces for any extension functions it includes.
+    xpath.callVisitors(xpath, new ExpressionVisitor(getStylesheetRoot()));
+    return xpath;
   }
 
   /**
@@ -203,11 +169,15 @@ public class StylesheetHandler extends DefaultHandler
    * @throws javax.xml.transform.TransformerException if the pattern can not be processed.
    * @see <a href="http://www.w3.org/TR/xslt#patterns">Section 5.2 Patterns in XSLT Specification</a>
    */
-  XPath createMatchPatternXPath(String str)
+  XPath createMatchPatternXPath(String str, ElemTemplateElement owningTemplate)
           throws javax.xml.transform.TransformerException
   {
     ErrorListener handler = m_stylesheetProcessor.getErrorListener();
-    return new XPath(str, getLocator(), this, XPath.MATCH, handler);
+    XPath xpath = new XPath(str, owningTemplate, this, XPath.MATCH, handler, 
+        m_funcTable);
+    // Visit the expression, registering namespaces for any extension functions it includes.
+    xpath.callVisitors(xpath, new ExpressionVisitor(getStylesheetRoot()));
+    return xpath;    
   }
 
   /**
@@ -240,7 +210,7 @@ public class StylesheetHandler extends DefaultHandler
 
     // Don't need to support this here.  Return the current URI for the prefix,
     // ignoring the context.
-    assert(true, "can't process a context node in StylesheetHandler!");
+    assertion(true, "can't process a context node in StylesheetHandler!");
 
     return null;
   }
@@ -285,9 +255,10 @@ public class StylesheetHandler extends DefaultHandler
    * @return The stylesheet object that was created during
    * the SAX event process, or null if no stylesheet has
    * been created.
+   * 
+   * Author <a href="mailto:scott_boag@lotus.com">Scott Boag</a>
+   *
    *
-   * @version Alpha
-   * @author <a href="mailto:scott_boag@lotus.com">Scott Boag</a>
    */
   public Templates getTemplates()
   {
@@ -401,18 +372,22 @@ public class StylesheetHandler extends DefaultHandler
     XSLTElementProcessor elemProcessor = def.getProcessorFor(uri, localName);
 
     if (null == elemProcessor
+            && !(currentProcessor instanceof ProcessorStylesheetDoc)
             && ((null == getStylesheet()
                 || Double.valueOf(getStylesheet().getVersion()).doubleValue()
-                   > Constants.XSLTVERSUPPORTED) ||
-                currentProcessor instanceof ProcessorStylesheetElement))
+                   > Constants.XSLTVERSUPPORTED) 
+                ||(!uri.equals(Constants.S_XSLNAMESPACEURL) &&
+                            currentProcessor instanceof ProcessorStylesheetElement)
+                || getElemVersion() > Constants.XSLTVERSUPPORTED
+        ))
     {
       elemProcessor = def.getProcessorForUnknown(uri, localName);
     }
 
     if (null == elemProcessor)
-      error(rawName + " is not allowed in this position in the stylesheet!",
-            null);
-
+      error(XSLMessages.createMessage(XSLTErrorResources.ER_NOT_ALLOWED_IN_POSITION, new Object[]{rawName}),null);//rawName + " is not allowed in this position in the stylesheet!",
+            
+                
     return elemProcessor;
   }
 
@@ -493,21 +468,17 @@ public class StylesheetHandler extends DefaultHandler
       if (null != getStylesheetRoot())
       {
         if (0 == m_stylesheetLevel)
-          getStylesheetRoot().recompose();
-
-        // Resolve the result prefix tables in the elements.
-        if (null != getLastPoppedStylesheet())
-          getLastPoppedStylesheet().resolvePrefixTables();
+          getStylesheetRoot().recompose();        
       }
       else
-        throw new TransformerException("Did not find the stylesheet root!");
+        throw new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_NO_STYLESHEETROOT, null)); //"Did not find the stylesheet root!");
 
       XSLTElementProcessor elemProcessor = getCurrentProcessor();
 
       if (null != elemProcessor)
         elemProcessor.startNonText(this);
 
-      m_stylesheetLevel--;
+      m_stylesheetLevel--;			
       
       popSpaceHandling();
 
@@ -590,8 +561,6 @@ public class StylesheetHandler extends DefaultHandler
   /**
    * Receive notification of the start of an element.
    *
-   * @param name The element type name.
-   *
    * @param uri The Namespace URI, or an empty string.
    * @param localName The local name (without prefix), or empty string if not namespace processing.
    * @param rawName The qualified name (with prefix).
@@ -607,6 +576,7 @@ public class StylesheetHandler extends DefaultHandler
     nssupport.pushContext();
     
     int n = m_prefixMappings.size();
+
     for (int i = 0; i < n; i++) 
     {
       String prefix = (String)m_prefixMappings.elementAt(i++);
@@ -618,6 +588,26 @@ public class StylesheetHandler extends DefaultHandler
 
     m_elementID++;
 
+    // This check is currently done for all elements.  We should possibly consider
+    // limiting this check to xsl:stylesheet elements only since that is all it really
+    // applies to.  Also, it could be bypassed if m_shouldProcess is already true.
+    // In other words, the next two statements could instead look something like this:
+    // if (!m_shouldProcess)
+    // {
+    //   if (localName.equals(Constants.ELEMNAME_STYLESHEET_STRING) &&
+    //       url.equals(Constants.S_XSLNAMESPACEURL))
+    //   {
+    //     checkForFragmentID(attributes);
+    //     if (!m_shouldProcess)
+    //       return;
+    //   }
+    //   else
+    //     return;
+    // } 
+    // I didn't include this code statement at this time because in practice 
+    // it is a small performance hit and I was waiting to see if its absence
+    // caused a problem. - GLP
+
     checkForFragmentID(attributes);
 
     if (!m_shouldProcess)
@@ -630,16 +620,22 @@ public class StylesheetHandler extends DefaultHandler
     XSLTElementProcessor elemProcessor = getProcessorFor(uri, localName,
                                            rawName);
 
-    this.pushProcessor(elemProcessor);
-    elemProcessor.startElement(this, uri, localName, rawName, attributes);
+    if(null != elemProcessor)  // defensive, for better multiple error reporting. -sb
+    {
+      this.pushProcessor(elemProcessor);
+      elemProcessor.startElement(this, uri, localName, rawName, attributes);
+    }
+    else
+    {
+      m_shouldProcess = false;
+      popSpaceHandling();
+    }
+                
   }
 
   /**
    * Receive notification of the end of an element.
    *
-   * @param name The element type name.
-   * @param attributes The specified or defaulted attributes.
-   *
    * @param uri The Namespace URI, or an empty string.
    * @param localName The local name (without prefix), or empty string if not namespace processing.
    * @param rawName The qualified name (with prefix).
@@ -702,8 +698,8 @@ public class StylesheetHandler extends DefaultHandler
       // If it's whitespace, just ignore it, otherwise flag an error.
       if (!XMLCharacterRecognizer.isWhiteSpace(ch, start, length))
         error(
-          "Non-whitespace text is not allowed in this position in the stylesheet!",
-          null);
+          XSLMessages.createMessage(XSLTErrorResources.ER_NONWHITESPACE_NOT_ALLOWED_IN_POSITION, null),null);//"Non-whitespace text is not allowed in this position in the stylesheet!",
+          
     }
     else
       elemProcessor.characters(this, ch, start, length);
@@ -734,6 +730,14 @@ public class StylesheetHandler extends DefaultHandler
   /**
    * Receive notification of a processing instruction.
    *
+   * <p>The Parser will invoke this method once for each processing
+   * instruction found: note that processing instructions may occur
+   * before or after the main document element.</p>
+   *
+   * <p>A SAX parser should never report an XML declaration (XML 1.0,
+   * section 2.8) or a text declaration (XML 1.0, section 4.3.1)
+   * using this method.</p>
+   *
    * <p>By default, do nothing.  Application writers may override this
    * method in a subclass to take specific actions for each
    * processing instruction, such as setting status variables or
@@ -750,10 +754,60 @@ public class StylesheetHandler extends DefaultHandler
   public void processingInstruction(String target, String data)
           throws org.xml.sax.SAXException
   {
-
     if (!m_shouldProcess)
       return;
 
+    // Recreating Scott's kluge:
+    // A xsl:for-each or xsl:apply-templates may have a special 
+    // PI that tells us not to cache the document.  This PI 
+    // should really be namespaced.
+    //    String localName = getLocalName(target);
+    //    String ns = m_stylesheet.getNamespaceFromStack(target);
+    //
+    // %REVIEW%: We need a better PI architecture
+    
+    String prefix="",ns="", localName=target;
+    int colon=target.indexOf(':');
+    if(colon>=0)
+    {
+      ns=getNamespaceForPrefix(prefix=target.substring(0,colon));
+      localName=target.substring(colon+1);
+    }
+
+    try
+    {
+      // A xsl:for-each or xsl:apply-templates may have a special 
+      // PI that tells us not to cache the document.  This PI 
+      // should really be namespaced... but since the XML Namespaces
+      // spec never defined namespaces as applying to PI's, and since
+      // the testcase we're trying to support is inconsistant in whether
+      // it binds the prefix, I'm going to make this sloppy for
+      // testing purposes.
+      if(
+        "xalan-doc-cache-off".equals(target) ||
+        "xalan:doc-cache-off".equals(target) ||
+	   ("doc-cache-off".equals(localName) &&
+	    ns.equals("org.apache.xalan.xslt.extensions.Redirect") )
+	 )
+      {
+	if(!(m_elems.peek() instanceof ElemForEach))
+          throw new TransformerException
+	    ("xalan:doc-cache-off not allowed here!", 
+	     getLocator());
+        ElemForEach elem = (ElemForEach)m_elems.peek();
+
+        elem.m_doc_cache_off = true;
+
+	//System.out.println("JJK***** Recognized <? {"+ns+"}"+prefix+":"+localName+" "+data+"?>");
+      }
+    }
+    catch(Exception e)
+    {
+      // JJK: Officially, unknown PIs can just be ignored.
+      // Do we want to issue a warning?
+    }
+
+
     flushCharacters();
     getCurrentProcessor().processingInstruction(this, target, data);
   }
@@ -782,10 +836,9 @@ public class StylesheetHandler extends DefaultHandler
   }
 
   /**
-   * <meta name="usage" content="internal"/>
    * Warn the user of an problem.
    *
-   * @param msg An index into the {@link org.apache.xalan.res.XSLTErrorResources}
+   * @param msg An key into the {@link org.apache.xalan.res.XSLTErrorResources}
    * table, that is one of the WG_ prefixed definitions.
    * @param args An array of arguments for the given warning.
    *
@@ -793,11 +846,12 @@ public class StylesheetHandler extends DefaultHandler
    * {@link javax.xml.transform.TransformerException} if the current
    * {@link javax.xml.transform.ErrorListener#warning}
    * method chooses to flag this condition as an error.
+   * @xsl.usage internal
    */
-  public void warn(int msg, Object args[]) throws org.xml.sax.SAXException
+  public void warn(String msg, Object args[]) throws org.xml.sax.SAXException
   {
 
-    String formattedMsg = m_XSLMessages.createWarning(msg, args);
+    String formattedMsg = XSLMessages.createWarning(msg, args);
     SAXSourceLocator locator = getLocator();
     ErrorListener handler = m_stylesheetProcessor.getErrorListener();
 
@@ -813,22 +867,21 @@ public class StylesheetHandler extends DefaultHandler
   }
 
   /**
-   * <meta name="usage" content="internal"/>
    * Assert that a condition is true.  If it is not true, throw an error.
    *
    * @param condition false if an error should not be thrown, otherwise true.
    * @param msg Error message to be passed to the RuntimeException as an
    * argument.
    * @throws RuntimeException if the condition is not true.
+   * @xsl.usage internal
    */
-  private void assert(boolean condition, String msg) throws RuntimeException
+  private void assertion(boolean condition, String msg) throws RuntimeException
   {
     if (!condition)
       throw new RuntimeException(msg);
   }
 
   /**
-   * <meta name="usage" content="internal"/>
    * Tell the user of an error, and probably throw an
    * exception.
    *
@@ -839,6 +892,7 @@ public class StylesheetHandler extends DefaultHandler
    * {@link javax.xml.transform.TransformerException} if the current
    * {@link javax.xml.transform.ErrorListener#error}
    * method chooses to flag this condition as an error.
+   * @xsl.usage internal
    */
   protected void error(String msg, Exception e)
           throws org.xml.sax.SAXException
@@ -873,11 +927,10 @@ public class StylesheetHandler extends DefaultHandler
   }
 
   /**
-   * <meta name="usage" content="internal"/>
    * Tell the user of an error, and probably throw an
    * exception.
    *
-   * @param msg An index into the {@link org.apache.xalan.res.XSLTErrorResources}
+   * @param msg A key into the {@link org.apache.xalan.res.XSLTErrorResources}
    * table, that is one of the WG_ prefixed definitions.
    * @param args An array of arguments for the given warning.
    * @param e An error which the SAXException should wrap.
@@ -886,12 +939,13 @@ public class StylesheetHandler extends DefaultHandler
    * {@link javax.xml.transform.TransformerException} if the current
    * {@link javax.xml.transform.ErrorListener#error}
    * method chooses to flag this condition as an error.
+   * @xsl.usage internal
    */
-  protected void error(int msg, Object args[], Exception e)
+  protected void error(String msg, Object args[], Exception e)
           throws org.xml.sax.SAXException
   {
 
-    String formattedMsg = m_XSLMessages.createMessage(msg, args);
+    String formattedMsg = XSLMessages.createMessage(msg, args);
 
     error(formattedMsg, e);
   }
@@ -1027,9 +1081,9 @@ public class StylesheetHandler extends DefaultHandler
 
         for (int i = 0; i < n; i++)
         {
-          String type = attributes.getType(i);
+          String name = attributes.getQName(i);
 
-          if (type.equalsIgnoreCase("ID"))
+          if (name.equals(Constants.ATTRNAME_ID))
           {
             String val = attributes.getValue(i);
 
@@ -1055,28 +1109,31 @@ public class StylesheetHandler extends DefaultHandler
    *
    * @return The TransformerFactoryImpl that owns this handler.
    */
-  TransformerFactoryImpl getStylesheetProcessor()
+  public TransformerFactoryImpl getStylesheetProcessor()
   {
     return m_stylesheetProcessor;
   }
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is a root stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_ROOT = 1;
+  public static final int STYPE_ROOT = 1;
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is an included stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_INCLUDE = 2;
+  public static final int STYPE_INCLUDE = 2;
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is an imported stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_IMPORT = 3;
+  public static final int STYPE_IMPORT = 3;
 
   /** The current stylesheet type. */
   private int m_stylesheetType = STYPE_ROOT;
@@ -1137,13 +1194,18 @@ public class StylesheetHandler extends DefaultHandler
    */
   public StylesheetRoot getStylesheetRoot()
   {
+    if (m_stylesheetRoot != null){
+        m_stylesheetRoot.setOptimizer(m_optimize);
+        m_stylesheetRoot.setIncremental(m_incremental);
+        m_stylesheetRoot.setSource_location(m_source_location);  		
+    }
     return m_stylesheetRoot;
   }
 
   /** The root stylesheet of the stylesheets tree. */
   StylesheetRoot m_stylesheetRoot;
-
-  /** The last stylesheet that was popped off the stylesheets stack. */
+        
+        /** The last stylesheet that was popped off the stylesheets stack. */
   Stylesheet m_lastPoppedStylesheet;
 
   /**
@@ -1232,8 +1294,9 @@ public class StylesheetHandler extends DefaultHandler
    * transition content handlers, create elements, etc.
    *
    * @return The root XSLT Schema, which should never be null.
+   * @xsl.usage internal
    */
-  XSLTSchema getSchema()
+  public XSLTSchema getSchema()
   {
     return m_schema;
   }
@@ -1258,7 +1321,7 @@ public class StylesheetHandler extends DefaultHandler
     {
       return null;
     }
-  }
+  }  
 
   /** An increasing number that is used to indicate the order in which this element
    *  was encountered during the parse of the XSLT tree.
@@ -1299,28 +1362,6 @@ public class StylesheetHandler extends DefaultHandler
     return (ElemTemplateElement) m_elems.pop();
   }
 
-  /**
-   * Flag to let us know when we've found an element inside the
-   * stylesheet that is not an xsl:import, so we can restrict imports
-   * to being the first elements.
-   */
-  private boolean m_foundNotImport = false;
-
-  /**
-   * A XSLMessages instance capable of producing user messages.
-   */
-  private static XSLMessages m_XSLMessages = new XSLMessages();
-
-  /**
-   * Get an XSLMessages instance capable of producing user messages.
-   *
-   * @return non-null reference to the error and warnings table.
-   */
-  XSLMessages getXSLMessages()
-  {
-    return m_XSLMessages;
-  }
-
   /**
    * This will act as a stack to keep track of the
    * current include base.
@@ -1423,6 +1464,13 @@ public class StylesheetHandler extends DefaultHandler
    * used to diagnose circular imports.
    */
   private Stack m_importStack = new Stack();
+  
+  /**
+   * A stack of Source objects obtained from a URIResolver,
+   * for each element in this stack there is a 1-1 correspondence
+   * with an element in the m_importStack.
+   */
+  private Stack m_importSourceStack = new Stack();
 
   /**
    * Push an import href onto the stylesheet stack.
@@ -1434,6 +1482,16 @@ public class StylesheetHandler extends DefaultHandler
   {
     m_importStack.push(hrefUrl);
   }
+  
+  /**
+   * Push the Source of an import href onto the stylesheet stack,
+   * obtained from a URIResolver, null if there is no URIResolver,
+   * or if that resolver returned null.
+   */
+  void pushImportSource(Source sourceFromURIResolver)
+  {
+    m_importSourceStack.push(sourceFromURIResolver);
+  }
 
   /**
    * See if the imported stylesheet stack already contains
@@ -1457,6 +1515,25 @@ public class StylesheetHandler extends DefaultHandler
   {
     return (String) m_importStack.pop();
   }
+  
+  String peekImportURL()
+  {
+    return (String) m_importStack.peek();
+  }
+  
+  Source peekSourceFromURIResolver()
+  {
+    return (Source) m_importSourceStack.peek();
+  }
+  
+  /**
+   * Pop a Source from a user provided URIResolver, corresponding
+   * to the URL popped from the m_importStack.
+   */
+  Source popImportSource()
+  {
+    return (Source) m_importSourceStack.pop();
+  }
 
   /**
    * If this is set to true, we've already warned about using the
@@ -1464,19 +1541,19 @@ public class StylesheetHandler extends DefaultHandler
    */
   private boolean warnedAboutOldXSLTNamespace = false;
 
-  /** Stack of {@link org.xml.sax.helpers.NamespaceSupport} objects. */
+  /** Stack of NamespaceSupport objects. */
   Stack m_nsSupportStack = new Stack();
 
   /**
-   * Push a new {@link org.xml.sax.helpers.NamespaceSupport} instance.
+   * Push a new NamespaceSupport instance.
    */
   void pushNewNamespaceSupport()
   {
-    m_nsSupportStack.push(new NamespaceSupport());
+    m_nsSupportStack.push(new NamespaceSupport2());
   }
 
   /**
-   * Pop the current {@link org.xml.sax.helpers.NamespaceSupport} object.
+   * Pop the current NamespaceSupport object.
    *
    */
   void popNamespaceSupport()
@@ -1572,7 +1649,7 @@ public class StylesheetHandler extends DefaultHandler
     String value = attrs.getValue("xml:space");
     if(null == value)
     {
-      m_spacePreserveStack.push(m_spacePreserveStack.peek());
+      m_spacePreserveStack.push(m_spacePreserveStack.peekOrFalse());
     }
     else if(value.equals("preserve"))
     {
@@ -1589,7 +1666,7 @@ public class StylesheetHandler extends DefaultHandler
   
       try
       {
-        handler.error(new TransformerException("Illegal value for xml:space", locator));
+        handler.error(new TransformerException(XSLMessages.createMessage(XSLTErrorResources.ER_ILLEGAL_XMLSPACE_VALUE, null), locator)); //"Illegal value for xml:space", locator));
       }
       catch (TransformerException te)
       {
@@ -1598,6 +1675,52 @@ public class StylesheetHandler extends DefaultHandler
       m_spacePreserveStack.push(m_spacePreserveStack.peek());
     }
   }
+  
+  private double getElemVersion()
+  {
+    ElemTemplateElement elem = getElemTemplateElement();
+    double version = -1; 
+    while ((version == -1 || version == Constants.XSLTVERSUPPORTED) && elem != null)
+    {
+      try{
+      version = Double.valueOf(elem.getXmlVersion()).doubleValue();
+      }
+      catch (Exception ex)
+      {
+        version = -1;
+      }
+      elem = elem.getParentElem();
+      }
+    return (version == -1)? Constants.XSLTVERSUPPORTED : version;
+  }
+    /**
+     * @see PrefixResolver#handlesNullPrefixes()
+     */
+    public boolean handlesNullPrefixes() {
+        return false;
+    }
+
+    /**
+     * @return Optimization flag
+     */
+    public boolean getOptimize() {
+        return m_optimize;
+    }
+
+    /**
+     * @return Incremental flag
+     */
+    public boolean getIncremental() {
+        return m_incremental;
+    }
+
+    /**
+     * @return Source Location flag
+     */
+    public boolean getSource_location() {
+        return m_source_location;
+    }
+
 }
 
 
