@@ -1,108 +1,69 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: TransformerFactoryImpl.java 468640 2006-10-28 06:53:53Z minchau $
  */
 package org.apache.xalan.processor;
 
-import org.xml.sax.InputSource;
-import org.xml.sax.helpers.XMLReaderFactory;
-import org.xml.sax.XMLReader;
+import java.io.BufferedInputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.Enumeration;
+import java.util.Properties;
 
+import javax.xml.XMLConstants;
+import javax.xml.transform.ErrorListener;
+import javax.xml.transform.Source;
+import javax.xml.transform.Templates;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerConfigurationException;
 import javax.xml.transform.TransformerException;
+import javax.xml.transform.URIResolver;
+import javax.xml.transform.dom.DOMResult;
+import javax.xml.transform.dom.DOMSource;
+import javax.xml.transform.sax.SAXResult;
+import javax.xml.transform.sax.SAXSource;
+import javax.xml.transform.sax.SAXTransformerFactory;
+import javax.xml.transform.sax.TemplatesHandler;
+import javax.xml.transform.sax.TransformerHandler;
+import javax.xml.transform.stream.StreamResult;
+import javax.xml.transform.stream.StreamSource;
 
-import org.xml.sax.SAXNotRecognizedException;
-import org.xml.sax.SAXNotSupportedException;
-import org.xml.sax.XMLFilter;
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+import org.apache.xalan.transformer.TrAXFilter;
+import org.apache.xalan.transformer.TransformerIdentityImpl;
+import org.apache.xalan.transformer.TransformerImpl;
+import org.apache.xalan.transformer.XalanProperties;
+
+import org.apache.xml.dtm.ref.sax2dtm.SAX2DTM;
+import org.apache.xml.utils.DefaultErrorHandler;
+import org.apache.xml.utils.SystemIDResolver;
+import org.apache.xml.utils.TreeWalker;
+import org.apache.xml.utils.StylesheetPIHandler;
+import org.apache.xml.utils.StopParseException;
 
 import org.w3c.dom.Node;
 
-import org.apache.xml.utils.TreeWalker;
-import org.apache.xml.utils.SystemIDResolver;
-import org.apache.xml.utils.DefaultErrorHandler;
-import org.apache.xalan.transformer.TransformerImpl;
-import org.apache.xalan.transformer.TransformerIdentityImpl;
-import org.apache.xalan.transformer.TrAXFilter;
-import org.apache.xalan.res.XSLMessages;
-import org.apache.xalan.res.XSLTErrorResources;
-
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.TransformerConfigurationException;
-import javax.xml.transform.Source;
-import javax.xml.transform.URIResolver;
-import javax.xml.transform.Templates;
-import javax.xml.transform.sax.TemplatesHandler;
-import javax.xml.transform.sax.TransformerHandler;
-import javax.xml.transform.sax.SAXTransformerFactory;
-import javax.xml.transform.sax.SAXSource;
-import javax.xml.transform.sax.SAXResult;
-import javax.xml.transform.dom.DOMSource;
-import javax.xml.transform.dom.DOMResult;
-import javax.xml.transform.stream.StreamSource;
-import javax.xml.transform.stream.StreamResult;
-import javax.xml.transform.ErrorListener;
-
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.BufferedInputStream;
-import java.io.StringWriter;
-import java.io.PrintWriter;
-import java.io.StringReader;
-
-import java.util.Properties;
-import java.util.Enumeration;
+import org.xml.sax.InputSource;
+import org.xml.sax.XMLFilter;
+import org.xml.sax.XMLReader;
+import org.xml.sax.helpers.XMLReaderFactory;
 
 /**
  * The TransformerFactoryImpl, which implements the TRaX TransformerFactory
@@ -111,13 +72,18 @@ import java.util.Enumeration;
  */
 public class TransformerFactoryImpl extends SAXTransformerFactory
 {
+  /** 
+   * The path/filename of the property file: XSLTInfo.properties  
+   * Maintenance note: see also
+   * <code>org.apache.xpath.functions.FuncSystemProperty.XSLT_PROPERTIES</code>
+   */
+  public static final String XSLT_PROPERTIES =
+    "org/apache/xalan/res/XSLTInfo.properties";
 
-  /** The Xalan properties file. */
-  public static String XSLT_PROPERTIES =
-    "/org/apache/xalan/res/XSLTInfo.properties";
-
-  /** Flag tells if the properties file has been loaded to the system */
-  private static boolean isInited = false;
+  /**
+   * <p>State of secure processing feature.</p>
+   */
+  private boolean m_isSecureProcessing = false;
 
   /**
    * Constructor TransformerFactoryImpl
@@ -125,72 +91,30 @@ public class TransformerFactoryImpl extends SAXTransformerFactory
    */
   public TransformerFactoryImpl()
   {
-    loadPropertyFileToSystem(XSLT_PROPERTIES);
   }
 
-  /**
-   * Retrieve a propery bundle from a specified file and load it
-   * int the System properties.
-   *
-   * @param file The properties file to be processed.
-   */
-  private static void loadPropertyFileToSystem(String file)
-  {
+  /** Static string to be used for incremental feature */
+  public static final String FEATURE_INCREMENTAL =
+                             "http://xml.apache.org/xalan/features/incremental";
 
-    if (false == isInited)
-    {
-      try
-      {
-        InputStream is;
+  /** Static string to be used for optimize feature */
+  public static final String FEATURE_OPTIMIZE =
+                             "http://xml.apache.org/xalan/features/optimize";
 
-        try
-        {
-          Properties props = new Properties();
+  /** Static string to be used for source_location feature */
+  public static final String FEATURE_SOURCE_LOCATION =
+                             XalanProperties.SOURCE_LOCATION;
 
-          is = Process.class.getResourceAsStream(file);
-
-          // get a buffered version
-          BufferedInputStream bis = new BufferedInputStream(is);
-
-          props.load(bis);  // and load up the property bag from this
-          bis.close();  // close out after reading
-
-          // OK, now we only want to set system properties that 
-          // are not already set.
-          Properties systemProps = System.getProperties();
-          Enumeration propEnum = props.propertyNames();
-
-          while (propEnum.hasMoreElements())
-          {
-            String prop = (String) propEnum.nextElement();
-
-            if (!systemProps.containsKey(prop))
-              systemProps.put(prop, props.getProperty(prop));
-          }
-
-          System.setProperties(systemProps);
-
-          isInited = true;
-        }
-        catch (Exception ex){}
-      }
-      catch (SecurityException se)
-      {
-
-        // In this case the caller is required to have 
-        // the needed attributes already defined.
-      }
-    }
-  }
-
-public javax.xml.transform.Templates processFromNode(Node node)
+  public javax.xml.transform.Templates processFromNode(Node node)
           throws TransformerConfigurationException
   {
 
     try
     {
       TemplatesHandler builder = newTemplatesHandler();
-      TreeWalker walker = new TreeWalker(builder, new org.apache.xpath.DOM2Helper());
+      TreeWalker walker = new TreeWalker(builder,
+                                         new org.apache.xml.utils.DOM2Helper(),
+                                         builder.getSystemId());
 
       walker.traverse(node);
 
@@ -204,6 +128,10 @@ public javax.xml.transform.Templates processFromNode(Node node)
         {
           m_errorListener.fatalError(new TransformerException(se));
         }
+        catch (TransformerConfigurationException ex)
+        {
+          throw ex;
+        }
         catch (TransformerException ex)
         {
           throw new TransformerConfigurationException(ex);
@@ -212,23 +140,25 @@ public javax.xml.transform.Templates processFromNode(Node node)
         return null;
       }
       else
+      {
 
         // Should remove this later... but right now diagnostics from 
         // TransformerConfigurationException are not good.
         // se.printStackTrace();
-        throw new TransformerConfigurationException("processFromNode failed",
-                                                    se);
+        throw new TransformerConfigurationException(XSLMessages.createMessage(XSLTErrorResources.ER_PROCESSFROMNODE_FAILED, null), se); 
+        //"processFromNode failed", se);
+      }
     }
     catch (TransformerConfigurationException tce)
     {
       // Assume it's already been reported to the error listener.
       throw tce;
     }
-    catch (TransformerException tce)
+   /* catch (TransformerException tce)
     {
       // Assume it's already been reported to the error listener.
       throw new TransformerConfigurationException(tce.getMessage(), tce);
-    }
+    }*/
     catch (Exception e)
     {
       if (m_errorListener != null)
@@ -237,6 +167,10 @@ public javax.xml.transform.Templates processFromNode(Node node)
         {
           m_errorListener.fatalError(new TransformerException(e));
         }
+        catch (TransformerConfigurationException ex)
+        {
+          throw ex;
+        }
         catch (TransformerException ex)
         {
           throw new TransformerConfigurationException(ex);
@@ -245,12 +179,13 @@ public javax.xml.transform.Templates processFromNode(Node node)
         return null;
       }
       else
-
+      {
         // Should remove this later... but right now diagnostics from 
         // TransformerConfigurationException are not good.
         // se.printStackTrace();
-        throw new TransformerConfigurationException("processFromNode failed",
-                                                    e);
+        throw new TransformerConfigurationException(XSLMessages.createMessage(XSLTErrorResources.ER_PROCESSFROMNODE_FAILED, null), e); //"processFromNode failed",
+                                                    //e);
+      }
     }
   }
 
@@ -356,7 +291,7 @@ public javax.xml.transform.Templates processFromNode(Node node)
     {
       if (null != node)
       {
-        TreeWalker walker = new TreeWalker(handler, new org.apache.xpath.DOM2Helper());
+        TreeWalker walker = new TreeWalker(handler, new org.apache.xml.utils.DOM2Helper(), baseID);
 
         walker.traverse(node);
       }
@@ -371,6 +306,15 @@ public javax.xml.transform.Templates processFromNode(Node node)
 
           factory.setNamespaceAware(true);
 
+          if (m_isSecureProcessing)
+          {
+            try
+            {
+              factory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
+            }
+            catch (org.xml.sax.SAXException e) {}
+          }
+
           javax.xml.parsers.SAXParser jaxpParser = factory.newSAXParser();
 
           reader = jaxpParser.getXMLReader();
@@ -419,8 +363,6 @@ public javax.xml.transform.Templates processFromNode(Node node)
    * Create a new Transformer object that performs a copy
    * of the source to the result.
    *
-   * @param source An object that holds a URI, input stream, etc.
-   *
    * @return A Transformer object that may be used to perform a transformation
    * in a single thread, never null.
    *
@@ -434,9 +376,54 @@ public javax.xml.transform.Templates processFromNode(Node node)
     return new StylesheetHandler(this);
   }
 
+  /**
+   * <p>Set a feature for this <code>TransformerFactory</code> and <code>Transformer</code>s
+   * or <code>Template</code>s created by this factory.</p>
+   * 
+   * <p>
+   * Feature names are fully qualified {@link java.net.URI}s.
+   * Implementations may define their own features.
+   * An {@link TransformerConfigurationException} is thrown if this <code>TransformerFactory</code> or the
+   * <code>Transformer</code>s or <code>Template</code>s it creates cannot support the feature.
+   * It is possible for an <code>TransformerFactory</code> to expose a feature value but be unable to change its state.
+   * </p>
+   * 
+   * <p>See {@link javax.xml.transform.TransformerFactory} for full documentation of specific features.</p>
+   * 
+   * @param name Feature name.
+   * @param value Is feature state <code>true</code> or <code>false</code>.
+   *  
+   * @throws TransformerConfigurationException if this <code>TransformerFactory</code>
+   *   or the <code>Transformer</code>s or <code>Template</code>s it creates cannot support this feature.
+   * @throws NullPointerException If the <code>name</code> parameter is null.
+   */
+  public void setFeature(String name, boolean value)
+	  throws TransformerConfigurationException {
+  
+  	// feature name cannot be null
+  	if (name == null) {
+  	    throw new NullPointerException(
+                  XSLMessages.createMessage(
+                      XSLTErrorResources.ER_SET_FEATURE_NULL_NAME, null));    
+  	}
+  		
+  	// secure processing?
+  	if (name.equals(XMLConstants.FEATURE_SECURE_PROCESSING)) {
+  	    m_isSecureProcessing = value;			
+  	}
+  	// This implementation does not support the setting of a feature other than
+  	// the secure processing feature.
+  	else
+    {
+      throw new TransformerConfigurationException(
+          XSLMessages.createMessage(
+            XSLTErrorResources.ER_UNSUPPORTED_FEATURE, 
+            new Object[] {name}));
+    }
+  }
+
   /**
    * Look up the value of a feature.
-   *
    * <p>The feature name is any fully-qualified URI.  It is
    * possible for an TransformerFactory to recognize a feature name but
    * to be unable to return its value; this is especially true
@@ -447,9 +434,16 @@ public javax.xml.transform.Templates processFromNode(Node node)
    * @param name The feature name, which is a fully-qualified URI.
    * @return The current state of the feature (true or false).
    */
-  public boolean getFeature(String name)
-  {
-
+  public boolean getFeature(String name) {
+  	
+    // feature name cannot be null
+    if (name == null) 
+    {
+    	throw new NullPointerException(
+            XSLMessages.createMessage(
+            XSLTErrorResources.ER_GET_FEATURE_NULL_NAME, null));    
+    }
+	  	
     // Try first with identity comparison, which 
     // will be faster.
     if ((DOMResult.FEATURE == name) || (DOMSource.FEATURE == name)
@@ -467,17 +461,48 @@ public javax.xml.transform.Templates processFromNode(Node node)
              || (StreamSource.FEATURE.equals(name))
              || (SAXTransformerFactory.FEATURE.equals(name))
              || (SAXTransformerFactory.FEATURE_XMLFILTER.equals(name)))
-      return true;
-    else
+      return true;	      
+    // secure processing?
+    else if (name.equals(XMLConstants.FEATURE_SECURE_PROCESSING))
+      return m_isSecureProcessing;
+    else      
+      // unknown feature
       return false;
   }
-
+  
+  /**
+   * Flag set by FEATURE_OPTIMIZE.
+   * This feature specifies whether to Optimize stylesheet processing. By
+   * default it is set to true.
+   */
+  private boolean m_optimize = true;
+  
+  /** Flag set by FEATURE_SOURCE_LOCATION.
+   * This feature specifies whether the transformation phase should
+   * keep track of line and column numbers for the input source
+   * document. Note that this works only when that
+   * information is available from the source -- in other words, if you
+   * pass in a DOM, there's little we can do for you.
+   * 
+   * The default is false. Setting it true may significantly
+   * increase storage cost per node. 
+   */
+  private boolean m_source_location = false;
+  
+  /**
+   * Flag set by FEATURE_INCREMENTAL.
+   * This feature specifies whether to produce output incrementally, rather than
+   * waiting to finish parsing the input before generating any output. By 
+   * default this attribute is set to false. 
+   */
+  private boolean m_incremental = false;
+  
   /**
    * Allows the user to set specific attributes on the underlying
    * implementation.
    *
    * @param name The name of the attribute.
-   * @param value The value of the attribute.
+   * @param value The value of the attribute; Boolean or String="true"|"false"
    *
    * @throws IllegalArgumentException thrown if the underlying
    * implementation doesn't recognize the attribute.
@@ -485,7 +510,71 @@ public javax.xml.transform.Templates processFromNode(Node node)
   public void setAttribute(String name, Object value)
           throws IllegalArgumentException
   {
-    throw new IllegalArgumentException(name);
+    if (name.equals(FEATURE_INCREMENTAL))
+    {
+      if(value instanceof Boolean)
+      {
+        // Accept a Boolean object..
+        m_incremental = ((Boolean)value).booleanValue();
+      }
+      else if(value instanceof String)
+      {
+        // .. or a String object
+        m_incremental = (new Boolean((String)value)).booleanValue();
+      }
+      else
+      {
+        // Give a more meaningful error message
+        throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_BAD_VALUE, new Object[]{name, value})); //name + " bad value " + value);
+      }
+	}
+    else if (name.equals(FEATURE_OPTIMIZE))
+    {
+      if(value instanceof Boolean)
+      {
+        // Accept a Boolean object..
+        m_optimize = ((Boolean)value).booleanValue();
+      }
+      else if(value instanceof String)
+      {
+        // .. or a String object
+        m_optimize = (new Boolean((String)value)).booleanValue();
+      }
+      else
+      {
+        // Give a more meaningful error message
+        throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_BAD_VALUE, new Object[]{name, value})); //name + " bad value " + value);
+      }
+    }
+    
+    // Custom Xalan feature: annotate DTM with SAX source locator fields.
+    // This gets used during SAX2DTM instantiation. 
+    //
+    // %REVIEW% Should the name of this field really be in XalanProperties?
+    // %REVIEW% I hate that it's a global static, but didn't want to change APIs yet.
+    else if(name.equals(FEATURE_SOURCE_LOCATION))
+    {
+      if(value instanceof Boolean)
+      {
+        // Accept a Boolean object..
+        m_source_location = ((Boolean)value).booleanValue();
+      }
+      else if(value instanceof String)
+      {
+        // .. or a String object
+        m_source_location = (new Boolean((String)value)).booleanValue();
+      }
+      else
+      {
+        // Give a more meaningful error message
+        throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_BAD_VALUE, new Object[]{name, value})); //name + " bad value " + value);
+      }
+    }
+    
+    else
+    {
+      throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_NOT_SUPPORTED, new Object[]{name})); //name + "not supported");
+    }
   }
 
   /**
@@ -500,7 +589,20 @@ public javax.xml.transform.Templates processFromNode(Node node)
    */
   public Object getAttribute(String name) throws IllegalArgumentException
   {
-    throw new IllegalArgumentException(name);
+    if (name.equals(FEATURE_INCREMENTAL))
+    {
+      return new Boolean(m_incremental);            
+    }
+    else if (name.equals(FEATURE_OPTIMIZE))
+    {
+      return new Boolean(m_optimize);
+    }
+    else if (name.equals(FEATURE_SOURCE_LOCATION))
+    {
+      return new Boolean(m_source_location);
+    }
+    else
+      throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_ATTRIB_VALUE_NOT_RECOGNIZED, new Object[]{name})); //name + " attribute not recognized");
   }
 
   /**
@@ -527,8 +629,6 @@ public javax.xml.transform.Templates processFromNode(Node node)
    * Create an XMLFilter that uses the given source as the
    * transformation instructions.
    *
-   * @param src The source of the transformation instructions.
-   *
    * @param templates non-null reference to Templates object.
    *
    * @return An XMLFilter object, or null if this feature is not supported.
@@ -538,15 +638,26 @@ public javax.xml.transform.Templates processFromNode(Node node)
   public XMLFilter newXMLFilter(Templates templates)
           throws TransformerConfigurationException
   {
-    try {
+    try 
+    {
       return new TrAXFilter(templates);
-    } catch( TransformerConfigurationException ex ) {
-      if( m_errorListener != null) {
-        try {
+    } 
+    catch( TransformerConfigurationException ex ) 
+    {
+      if( m_errorListener != null) 
+      {
+        try 
+        {
           m_errorListener.fatalError( ex );
           return null;
-        } catch( TransformerException ex1 ) {
-          new TransformerConfigurationException(ex1);
+        } 
+        catch( TransformerConfigurationException ex1 ) 
+        {
+          throw ex1;
+        }
+        catch( TransformerException ex1 ) 
+        {
+          throw new TransformerConfigurationException(ex1);
         }
       }
       throw ex;
@@ -589,19 +700,31 @@ public javax.xml.transform.Templates processFromNode(Node node)
     try {
       TransformerImpl transformer =
         (TransformerImpl) templates.newTransformer();
+      transformer.setURIResolver(m_uriResolver);
       TransformerHandler th =
         (TransformerHandler) transformer.getInputContentHandler(true);
 
       return th;
-    } catch( TransformerConfigurationException ex ) {
-      if( m_errorListener != null ) {
-        try {
+    } 
+    catch( TransformerConfigurationException ex ) 
+    {
+      if( m_errorListener != null ) 
+      {
+        try 
+        {
           m_errorListener.fatalError( ex );
           return null;
-        } catch (TransformerException ex1 ) {
-          ex=new TransformerConfigurationException(ex1);
+        } 
+        catch (TransformerConfigurationException ex1 ) 
+        {
+          throw ex1;
+        }
+        catch (TransformerException ex1 ) 
+        {
+          throw new TransformerConfigurationException(ex1);
         }
       }
+      
       throw ex;
     }
     
@@ -622,8 +745,6 @@ public javax.xml.transform.Templates processFromNode(Node node)
    * Get a TransformerHandler object that can process SAX
    * ContentHandler events into a Result.
    *
-   * @param src The source of the transformation instructions.
-   *
    * @return TransformerHandler ready to transform SAX events.
    *
    * @throws TransformerConfigurationException
@@ -631,22 +752,7 @@ public javax.xml.transform.Templates processFromNode(Node node)
   public TransformerHandler newTransformerHandler()
           throws TransformerConfigurationException
   {
-
-//    if (null == m_identityTemplate)
-//    {
-//      synchronized (identityTransform)
-//      {
-//        if (null == m_identityTemplate)
-//        {
-//          StringReader reader = new StringReader(identityTransform);
-//
-//          m_identityTemplate = newTemplates(new StreamSource(reader));
-//        }
-//      }
-//    }
-//
-//    return newTransformerHandler(m_identityTemplate);
-    return new TransformerIdentityImpl();
+    return new TransformerIdentityImpl(m_isSecureProcessing);
   }
 
   /**
@@ -665,7 +771,8 @@ public javax.xml.transform.Templates processFromNode(Node node)
   public Transformer newTransformer(Source source)
           throws TransformerConfigurationException
   {
-    try {
+    try 
+    {
       Templates tmpl=newTemplates( source );
       /* this can happen if an ErrorListener is present and it doesn't
          throw any exception in fatalError. 
@@ -674,14 +781,26 @@ public javax.xml.transform.Templates processFromNode(Node node)
          that, and returns null.
       */
       if( tmpl==null ) return null;
-      return tmpl.newTransformer();
-    } catch( TransformerConfigurationException ex ) {
-      if( m_errorListener != null ) {
-        try {
+      Transformer transformer = tmpl.newTransformer();
+      transformer.setURIResolver(m_uriResolver);
+      return transformer;
+    } 
+    catch( TransformerConfigurationException ex ) 
+    {
+      if( m_errorListener != null ) 
+      {
+        try 
+        {
           m_errorListener.fatalError( ex );
           return null;
-        } catch( TransformerException ex1 ) {
-          ex=new TransformerConfigurationException( ex1 );
+        } 
+        catch( TransformerConfigurationException ex1 ) 
+        {
+          throw ex1;
+        }
+        catch( TransformerException ex1 ) 
+        {
+          throw new TransformerConfigurationException( ex1 );
         }
       }
       throw ex;
@@ -692,8 +811,6 @@ public javax.xml.transform.Templates processFromNode(Node node)
    * Create a new Transformer object that performs a copy
    * of the source to the result.
    *
-   * @param source An object that holds a URL, input stream, etc.
-   *
    * @return A Transformer object capable of
    * being used for transformation purposes in a single thread.
    *
@@ -703,22 +820,7 @@ public javax.xml.transform.Templates processFromNode(Node node)
    */
   public Transformer newTransformer() throws TransformerConfigurationException
   {
-
-//    if (null == m_identityTemplate)
-//    {
-//      synchronized (identityTransform)
-//      {
-//        if (null == m_identityTemplate)
-//        {
-//          StringReader reader = new StringReader(identityTransform);
-//
-//          m_identityTemplate = newTemplates(new StreamSource(reader));
-//        }
-//      }
-//    }
-//
-//    return m_identityTemplate.newTransformer();
-      return new TransformerIdentityImpl();
+      return new TransformerIdentityImpl(m_isSecureProcessing);
   }
 
   /**
@@ -739,38 +841,12 @@ public javax.xml.transform.Templates processFromNode(Node node)
           throws TransformerConfigurationException
   {
 
-    TemplatesHandler builder = newTemplatesHandler();
     String baseID = source.getSystemId();
 
-    if (null == baseID)
-    {
-      try
-      {
-        String currentDir = System.getProperty("user.dir");
-
-        baseID = "file:///" + currentDir + java.io.File.separatorChar
-                 + source.getClass().getName();
-      }
-      catch (SecurityException se)
-      {
-
-        // For untrusted applet case, user.dir is outside the sandbox 
-        //  and not accessible: just leave baseID as null (-sb & -sc)
-      }
-    }
-    else
-    {
-      try
-      {
-        baseID = SystemIDResolver.getAbsoluteURI(baseID);
-      }
-      catch (TransformerException te)
-      {
-        throw new TransformerConfigurationException(te);
-      }
+    if (null != baseID) {
+       baseID = SystemIDResolver.getAbsoluteURI(baseID);
     }
 
-    builder.setSystemId(baseID);
 
     if (source instanceof DOMSource)
     {
@@ -788,14 +864,18 @@ public javax.xml.transform.Templates processFromNode(Node node)
       }
     }
 
+    TemplatesHandler builder = newTemplatesHandler();
+    builder.setSystemId(baseID);
+    
     try
     {
       InputSource isource = SAXSource.sourceToInputSource(source);
+      isource.setSystemId(baseID);
       XMLReader reader = null;
 
       if (source instanceof SAXSource)
         reader = ((SAXSource) source).getXMLReader();
-
+        
       if (null == reader)
       {
 
@@ -807,6 +887,15 @@ public javax.xml.transform.Templates processFromNode(Node node)
 
           factory.setNamespaceAware(true);
 
+          if (m_isSecureProcessing)
+          {
+            try
+            {
+              factory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
+            }
+            catch (org.xml.sax.SAXException se) {}
+          }
+
           javax.xml.parsers.SAXParser jaxpParser = factory.newSAXParser();
 
           reader = jaxpParser.getXMLReader();
@@ -829,17 +918,6 @@ public javax.xml.transform.Templates processFromNode(Node node)
       // If you set the namespaces to true, we'll end up getting double 
       // xmlns attributes.  Needs to be fixed.  -sb
       // reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
-      try
-      {
-        reader.setFeature("http://apache.org/xml/features/validation/dynamic",
-                          true);
-      }
-      catch (org.xml.sax.SAXException ex)
-      {
-
-        // feature not recognized
-      }
-
       reader.setContentHandler(builder);
       reader.parse(isource);
     }
@@ -851,13 +929,19 @@ public javax.xml.transform.Templates processFromNode(Node node)
         {
           m_errorListener.fatalError(new TransformerException(se));
         }
+        catch (TransformerConfigurationException ex1)
+        {
+          throw ex1;
+        }
         catch (TransformerException ex1)
         {
           throw new TransformerConfigurationException(ex1);
         }
       }
       else
+      {
         throw new TransformerConfigurationException(se.getMessage(), se);
+      }
     }
     catch (Exception e)
     {
@@ -866,16 +950,21 @@ public javax.xml.transform.Templates processFromNode(Node node)
         try
         {
           m_errorListener.fatalError(new TransformerException(e));
-
           return null;
         }
+        catch (TransformerConfigurationException ex1)
+        {
+          throw ex1;
+        }
         catch (TransformerException ex1)
         {
           throw new TransformerConfigurationException(ex1);
         }
       }
       else
+      {
         throw new TransformerConfigurationException(e.getMessage(), e);
+      }
     }
 
     return builder.getTemplates();
@@ -912,7 +1001,7 @@ public javax.xml.transform.Templates processFromNode(Node node)
   }
 
   /** The error listener.   */
-  private ErrorListener m_errorListener = new DefaultErrorHandler();
+  private ErrorListener m_errorListener = new org.apache.xml.utils.DefaultErrorHandler(false);
 
   /**
    * Get the error listener in effect for the TransformerFactory.
@@ -936,8 +1025,19 @@ public javax.xml.transform.Templates processFromNode(Node node)
   {
 
     if (null == listener)
-      throw new IllegalArgumentException("ErrorListener");
+      throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_ERRORLISTENER, null));
+      // "ErrorListener");
 
     m_errorListener = listener;
   }
+  
+  /**
+   * Return the state of the secure processing feature.
+   * 
+   * @return state of the secure processing feature.
+   */
+  public boolean isSecureProcessing()
+  {
+    return m_isSecureProcessing;
+  }
 }
