@@ -1,81 +1,38 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: OutputProperties.java 468643 2006-10-28 06:56:03Z minchau $
  */
 package org.apache.xalan.templates;
 
-import java.io.BufferedInputStream;
-import java.io.InputStream;
-import java.io.IOException;
-
-import java.util.Vector;
-import java.util.Hashtable;
-import java.util.Properties;
 import java.util.Enumeration;
+import java.util.Properties;
+import java.util.Vector;
 
-import java.lang.Cloneable;
-
-import org.w3c.dom.Document;
-
-import org.apache.xml.utils.QName;
-import org.apache.xml.utils.FastStringBuffer;
-import org.apache.xml.utils.WrappedRuntimeException;
-import org.apache.xalan.serialize.Method;
-
-import javax.xml.transform.TransformerException;
 import javax.xml.transform.OutputKeys;
+import javax.xml.transform.TransformerException;
+
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+import org.apache.xml.serializer.OutputPropertiesFactory;
+import org.apache.xml.serializer.OutputPropertyUtils;
+import org.apache.xml.utils.FastStringBuffer;
+import org.apache.xml.utils.QName;
 
 /**
  * This class provides information from xsl:output elements. It is mainly
@@ -87,18 +44,18 @@ import javax.xml.transform.OutputKeys;
  * is not found in the original property list.</p>
  * @see <a href="http://www.w3.org/TR/xslt#dtd">XSLT DTD</a>
  * @see <a href="http://www.w3.org/TR/xslt#output">xsl:output in XSLT Specification</a>
- * @
+ *
  */
 public class OutputProperties extends ElemTemplateElement
         implements Cloneable
 {
-
+    static final long serialVersionUID = -6975274363881785488L;
   /**
    * Creates an empty OutputProperties with no default values.
    */
   public OutputProperties()
   {
-    this(Method.XML);
+    this(org.apache.xml.serializer.Method.XML);
   }
 
   /**
@@ -123,187 +80,8 @@ public class OutputProperties extends ElemTemplateElement
    */
   public OutputProperties(String method)
   {
-    m_properties = new Properties(getDefaultMethodProperties(method));
-  }
-  
-  static final String S_XSLT_PREFIX = "xslt.output.";
-  static final int S_XSLT_PREFIX_LEN = S_XSLT_PREFIX.length();
-  static final String S_XALAN_PREFIX = "org.apache.xslt.";
-  static final int S_XALAN_PREFIX_LEN = S_XALAN_PREFIX.length();
-  
-  /** Built-in extensions namespace, reexpressed in {namespaceURI} syntax
-   * suitable for prepending to a localname to produce a "universal
-   * name".
-   */
-  static final String S_BUILTIN_EXTENSIONS_UNIVERSAL=
-        "{"+Constants.S_BUILTIN_EXTENSIONS_URL+"}";
-  
-  /**
-   * Fix up a string in an output properties file according to 
-   * the rules of {@link #loadPropertiesFile}.
-   * 
-   * @param s non-null reference to string that may need to be fixed up.
-   * @return A new string if fixup occured, otherwise the s argument.
-   */
-  static private String fixupPropertyString(String s, boolean doClipping)
-  {
-    int index;
-    if (doClipping && s.startsWith(S_XSLT_PREFIX))
-    {
-      s = s.substring(S_XSLT_PREFIX_LEN);
-    }
-    if (s.startsWith(S_XALAN_PREFIX))
-    {
-      s = S_BUILTIN_EXTENSIONS_UNIVERSAL + s.substring(S_XALAN_PREFIX_LEN);
-    }
-    if ((index = s.indexOf("\\u003a")) > 0)
-    {
-      String temp = s.substring(index+6);
-      s = s.substring(0, index) + ":" + temp;
-
-    }
-    return s;
-  }
-  
-  /**
-   * Load the properties file from a resource stream.  If a 
-   * key name such as "org.apache.xslt.xxx", fix up the start of 
-   * string to be a curly namespace.  If a key name starts with 
-   * "xslt.output.xxx", clip off "xslt.output.".  If a key name *or* a 
-   * key value is discovered, check for \u003a in the text, and 
-   * fix it up to be ":", since earlier versions of the JDK do not 
-   * handle the escape sequence (at least in key names).
-   * 
-   * @param resourceName non-null reference to resource name.
-   * @param defaults Default properties, which may be null.
-   */
-  static private Properties loadPropertiesFile(String resourceName, Properties defaults)
-    throws IOException
-  {
-    Properties props = new Properties(defaults);
-
-    InputStream is = OutputProperties.class.getResourceAsStream(
-                                                                resourceName);
-    BufferedInputStream bis = new BufferedInputStream(is);
-
-    props.load(bis);
-    
-    // Note that we're working at the HashTable level here, 
-    // and not at the Properties level!  This is important 
-    // because we don't want to modify the default properties.
-    Enumeration keys = props.keys();
-    while(keys.hasMoreElements())
-    {
-      String key = (String)keys.nextElement();
-      // Now check if the given key was specified as a 
-      // System property. If so, the system property 
-      // overides the default value in the propery file.
-      String value;
-      if ((value = System.getProperty(key)) == null)      
-        value = (String)props.get(key);                       
-      
-      String newKey = fixupPropertyString(key, true);
-      String newValue;
-      if ((newValue = System.getProperty(newKey)) == null)
-        newValue = fixupPropertyString(value, false);
-      else
-        newValue = fixupPropertyString(newValue, false);
-       
-      if(key != newKey || value != newValue)
-      {
-        props.remove(key);
-        props.put(newKey, newValue);
-      }
-      
-    }
-    
-    return props;
-  }
-
-  /**
-   * Creates an empty OutputProperties with the defaults specified by
-   * a property file.  The method argument is used to construct a string of
-   * the form output_[method].properties (for instance, output_html.properties).
-   * The output_xml.properties file is always used as the base.
-   * <p>At the moment, anything other than 'text', 'xml', and 'html', will
-   * use the output_xml.properties file.</p>
-   *
-   * @param   method non-null reference to method name.
-   *
-   * @return Properties object that holds the defaults for the given method.
-   */
-  static public Properties getDefaultMethodProperties(String method)
-  {
-
-    Properties defaultProperties = null;
-
-    try
-    {
-      if (null == m_xml_properties)  // fast check
-      {
-        synchronized (m_synch_object)
-        {
-          if (null == m_xml_properties)  // double check
-          {
-            m_xml_properties = loadPropertiesFile("output_xml.properties", null);
-          }
-        }
-      }
-
-      if (method.equals(Method.XML))
-      {
-        defaultProperties = m_xml_properties;
-      }
-      else if (method.equals(Method.HTML))
-      {
-        if (null == m_html_properties)  // fast check
-        {
-          synchronized (m_synch_object)
-          {
-            if (null == m_html_properties)  // double check
-            {
-              m_html_properties = loadPropertiesFile("output_html.properties", 
-                                                     m_xml_properties);
-            }
-          }
-        }
-
-        defaultProperties = m_html_properties;
-      }
-      else if (method.equals(Method.Text))
-      {
-        if (null == m_text_properties)  // fast check
-        {
-          synchronized (m_synch_object)
-          {
-            if (null == m_text_properties)  // double check
-            {
-              m_text_properties = loadPropertiesFile("output_text.properties", 
-                                                     null);
-              if(null == m_text_properties.getProperty(OutputKeys.ENCODING))
-              {
-                String mimeEncoding = org.apache.xalan.serialize.Encodings.getMimeEncoding(null);
-                m_text_properties.put(OutputKeys.ENCODING, mimeEncoding);
-              }
-            }
-          }
-        }
-
-        defaultProperties = m_text_properties;
-      }
-      else
-      {
-
-        // TODO: Calculate res file from name.
-        defaultProperties = m_xml_properties;
-      }
-    }
-    catch (IOException ioe)
-    {
-      throw new org.apache.xml.utils.WrappedRuntimeException(ioe);
-    }
-
-    return defaultProperties;
+    m_properties = new Properties(
+        OutputPropertiesFactory.getDefaultMethodProperties(method));
   }
 
   /**
@@ -355,6 +133,11 @@ public class OutputProperties extends ElemTemplateElement
     {
       setMethodDefaults(value);
     }
+    
+    if (key.startsWith(OutputPropertiesFactory.S_BUILTIN_OLD_EXTENSIONS_UNIVERSAL))
+      key = OutputPropertiesFactory.S_BUILTIN_EXTENSIONS_UNIVERSAL
+         + key.substring(OutputPropertiesFactory.S_BUILTIN_OLD_EXTENSIONS_UNIVERSAL_LEN);
+    
     m_properties.put(key, value);
   }
 
@@ -381,8 +164,11 @@ public class OutputProperties extends ElemTemplateElement
    * @param   key   the property key.
    * @return  the value in this property list with the specified key value.
    */
-  public String getProperty(String key)
+  public String getProperty(String key) 
   {
+    if (key.startsWith(OutputPropertiesFactory.S_BUILTIN_OLD_EXTENSIONS_UNIVERSAL))
+      key = OutputPropertiesFactory.S_BUILTIN_EXTENSIONS_UNIVERSAL 
+        + key.substring(OutputPropertiesFactory.S_BUILTIN_OLD_EXTENSIONS_UNIVERSAL_LEN);
     return m_properties.getProperty(key);
   }
 
@@ -439,32 +225,9 @@ public class OutputProperties extends ElemTemplateElement
    */
   public boolean getBooleanProperty(String key)
   {
-    return getBooleanProperty(key, m_properties);
+    return OutputPropertyUtils.getBooleanProperty(key, m_properties);
   }
 
-  /**
-   * Searches for the boolean property with the specified key in the property list.
-   * If the key is not found in this property list, the default property list,
-   * and its defaults, recursively, are then checked. The method returns
-   * <code>false</code> if the property is not found, or if the value is other
-   * than "yes".
-   *
-   * @param   key   the property key.
-   * @param   props   the list of properties that will be searched.
-   * @return  the value in this property list as a boolean value, or false
-   * if null or not "yes".
-   */
-  public static boolean getBooleanProperty(String key, Properties props)
-  {
-
-    String s = props.getProperty(key);
-
-    if (null == s ||!s.equals("yes"))
-      return false;
-    else
-      return true;
-  }
-  
   /**
    * Set an output property.
    *
@@ -518,31 +281,9 @@ public class OutputProperties extends ElemTemplateElement
    */
   public int getIntProperty(String key)
   {
-    return getIntProperty(key, m_properties);
+    return OutputPropertyUtils.getIntProperty(key, m_properties);
   }
 
-  /**
-   * Searches for the int property with the specified key in the property list.
-   * If the key is not found in this property list, the default property list,
-   * and its defaults, recursively, are then checked. The method returns
-   * <code>false</code> if the property is not found, or if the value is other
-   * than "yes".
-   *
-   * @param   key   the property key.
-   * @param   props   the list of properties that will be searched.
-   * @return  the value in this property list as a int value, or 0
-   * if null or not a number.
-   */
-  public static int getIntProperty(String key, Properties props)
-  {
-
-    String s = props.getProperty(key);
-
-    if (null == s)
-      return 0;
-    else
-      return Integer.parseInt(s);
-  }
 
   /**
    * Set an output property with a QName value.  The QName will be turned
@@ -565,14 +306,25 @@ public class OutputProperties extends ElemTemplateElement
    */
   public void setMethodDefaults(String method)
   {
-    String defaultMethod = m_properties.getProperty(OutputKeys.METHOD);
-    if((null == defaultMethod) || !defaultMethod.equals(method))
-    {
-      Properties savedProps = m_properties;
-      Properties newDefaults = getDefaultMethodProperties(method);
-      m_properties = new Properties(newDefaults);
-      copyFrom(savedProps, false);
-    }
+        String defaultMethod = m_properties.getProperty(OutputKeys.METHOD);
+ 
+        if((null == defaultMethod) || !defaultMethod.equals(method)
+         // bjm - add the next condition as a hack
+         // but it is because both output_xml.properties and
+         // output_unknown.properties have the same method=xml
+         // for their default. Otherwise we end up with
+         // a ToUnknownStream wraping a ToXMLStream even
+         // when the users says method="xml"
+         //
+         || defaultMethod.equals("xml")
+         )
+        {
+            Properties savedProps = m_properties;
+            Properties newDefaults = 
+                OutputPropertiesFactory.getDefaultMethodProperties(method);
+            m_properties = new Properties(newDefaults);
+            copyFrom(savedProps, false);
+        }
   }
   
 
@@ -797,12 +549,11 @@ public class OutputProperties extends ElemTemplateElement
    * values that may be based on some other property that
    * depends on recomposition.
    */
-  public void compose()
+  public void compose(StylesheetRoot sroot) throws TransformerException
   {
 
-    super.compose();  // just good form, not really needed.
+    super.compose(sroot);
 
-    m_propertiesLevels = null;
   }
 
   /**
@@ -841,11 +592,15 @@ public class OutputProperties extends ElemTemplateElement
   public void copyFrom(Properties src, boolean shouldResetDefaults)
   {
 
-    Enumeration enum = src.keys();
+    Enumeration keys = src.keys();
 
-    while (enum.hasMoreElements())
+    while (keys.hasMoreElements())
     {
-      String key = (String) enum.nextElement();
+      String key = (String) keys.nextElement();
+    
+      if (!isLegalPropertyKey(key))
+        throw new IllegalArgumentException(XSLMessages.createMessage(XSLTErrorResources.ER_OUTPUT_PROPERTY_NOT_RECOGNIZED, new Object[]{key})); //"output property not recognized: "
+      
       Object oldValue = m_properties.get(key);
       if (null == oldValue)
       {
@@ -876,63 +631,11 @@ public class OutputProperties extends ElemTemplateElement
   public void copyFrom(OutputProperties opsrc)
     throws TransformerException
   {
-    checkDuplicates(opsrc);
+   // Bugzilla 6157: recover from xsl:output statements
+    // checkDuplicates(opsrc);
     copyFrom(opsrc.getProperties());
   }
 
-  /**
-   * Check to see if a set of properties is at the same import level as the
-   * last set of properties set that was passed as an argument to this method.
-   * This operation assumes that the OutputProperties are being called
-   * from most important to least important, in document order.
-   *
-   * @param newProps non-null reference to OutputProperties that is about to
-   *                 be added to this set.
-   */
-  private void checkDuplicates(OutputProperties newProps)
-    throws TransformerException
-  {
-
-    if (null == m_propertiesLevels)
-      m_propertiesLevels = new Hashtable();
-
-    // This operation assumes that the OutputProperties are being called 
-    // from most important to least important, in reverse document order.
-
-    int newPrecedence = newProps.getStylesheetComposed().getImportCountComposed();
-
-    Properties p = newProps.getProperties();
-    Enumeration enum = p.keys();
-
-    while (enum.hasMoreElements())
-    {
-      String key = (String) enum.nextElement();
-
-      if (key.equals(OutputKeys.CDATA_SECTION_ELEMENTS))
-        continue;
-
-      // Do we already have this property? Call hashtable operation, 
-      // since we don't want to look at default properties.
-      Integer oldPrecedence = (Integer) m_propertiesLevels.get(key);
-      if (null == oldPrecedence)
-      {
-        m_propertiesLevels.put(key, new Integer(newPrecedence));
-      }
-      else if (newPrecedence >= oldPrecedence.intValue())
-      {
-        String oldValue = (String) this.m_properties.get(key);
-        String newValue = (String) newProps.m_properties.get(key);
-        if ( ((oldValue == null) && (newValue != null)) || !oldValue.equals(newValue) )
-        {
-          String msg = key + " can not be multiply defined at the same "
-                       + "import level! Old value = " 
-                       + oldValue + "; New value = " + newValue;
-          throw new TransformerException(msg, newProps);
-        }
-      }
-    }
-}
-
   /**
    * Report if the key given as an argument is a legal xsl:output key.
    *
@@ -940,7 +643,7 @@ public class OutputProperties extends ElemTemplateElement
    *
    * @return true if key is legal.
    */
-  public boolean isLegalPropertyKey(String key)
+  public static boolean isLegalPropertyKey(String key)
   {
 
     return (key.equals(OutputKeys.CDATA_SECTION_ELEMENTS)
@@ -953,54 +656,34 @@ public class OutputProperties extends ElemTemplateElement
             || key.equals(OutputKeys.OMIT_XML_DECLARATION)
             || key.equals(OutputKeys.STANDALONE)
             || key.equals(OutputKeys.VERSION)
-            || (key.length() > 0) && (key.charAt(0) == '{'));
+            || (key.length() > 0) 
+                  && (key.charAt(0) == '{') 
+                  && (key.lastIndexOf('{') == 0)
+                  && (key.indexOf('}') > 0)
+                  && (key.lastIndexOf('}') == key.indexOf('}')));
   }
 
-  /**
-   *  This ugly field is used during recomposition to track the import precedence
-   *  at which each attribute was first specified, so we can flag errors about values being
-   *  set multiple time at the same precedence level. Note that this field is only used
-   *  during recomposition, with the OutputProperties object owned by the
-   *  {@link org.apache.xalan.templates.StylesheetRoot} object.
-   */
-  private transient Hashtable m_propertiesLevels;
-
   /** The output properties.
    *  @serial */
   private Properties m_properties = null;
 
-  // Some special Xalan keys.
-
-  /** The number of whitespaces to indent by, if indent="yes". */
-  public static String S_KEY_INDENT_AMOUNT =
-    S_BUILTIN_EXTENSIONS_UNIVERSAL+"indent-amount";
-
-  /**
-   * Fully qualified name of class with a default constructor that
-   *  implements the ContentHandler interface, where the result tree events
-   *  will be sent to.      
-   */
-  public static String S_KEY_CONTENT_HANDLER =
-    S_BUILTIN_EXTENSIONS_UNIVERSAL+"content-handler";
-
-  /** File name of file that specifies character to entity reference mappings. */
-  public static String S_KEY_ENTITIES =
-    S_BUILTIN_EXTENSIONS_UNIVERSAL+"entities";
-
-  /** Use a value of "yes" if the href values for HTML serialization should 
-   *  use %xx escaping. */
-  public static String S_USE_URL_ESCAPING =
-    S_BUILTIN_EXTENSIONS_UNIVERSAL+"use-url-escaping";
-
-  /** The default properties of all output files. */
-  private static Properties m_xml_properties = null;
-
-  /** The default properties when method="html". */
-  private static Properties m_html_properties = null;
-
-  /** The default properties when method="text". */
-  private static Properties m_text_properties = null;
-
-  /** Synchronization object for lazy initialization of the above tables. */
-  private static Integer m_synch_object = new Integer(1);
+    /**
+     * Creates an empty OutputProperties with the defaults specified by
+     * a property file.  The method argument is used to construct a string of
+     * the form output_[method].properties (for instance, output_html.properties).
+     * The output_xml.properties file is always used as the base.
+     * <p>At the moment, anything other than 'text', 'xml', and 'html', will
+     * use the output_xml.properties file.</p>
+     *
+     * @param   method non-null reference to method name.
+     *
+     * @return Properties object that holds the defaults for the given method.
+     * 
+     * @deprecated Use org.apache.xml.serializer.OuputPropertiesFactory.
+     * getDefaultMethodProperties directly.
+     */
+    static public Properties getDefaultMethodProperties(String method)
+    {
+        return org.apache.xml.serializer.OutputPropertiesFactory.getDefaultMethodProperties(method);
+    }
 }
