@@ -1,99 +1,79 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: StylesheetRoot.java 476466 2006-11-18 08:22:31Z minchau $
  */
 package org.apache.xalan.templates;
 
-import org.w3c.dom.*;
-
-import java.util.*;
-
-import java.net.MalformedURLException;
-
-import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Hashtable;
+import java.util.Properties;
+import java.util.Vector;
 
-import java.io.*;
-
-import org.xml.sax.*;
-import org.xml.sax.helpers.*;
-
-import org.apache.xalan.serialize.*;
-import org.apache.xml.utils.*;
-import org.apache.xpath.*;
-import org.apache.xpath.compiler.XPathParser;
-import org.apache.xalan.trace.*;
-import org.apache.xalan.res.XSLTErrorResources;
-import org.apache.xalan.res.XSLMessages;
-import org.apache.xalan.processor.XSLTSchema;
-import org.apache.xalan.transformer.TransformerImpl;
-
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerConfigurationException;
-import javax.xml.transform.Templates;
-import javax.xml.transform.OutputKeys;
 import javax.xml.transform.ErrorListener;
+import javax.xml.transform.Templates;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerConfigurationException;
+import javax.xml.transform.TransformerException;
+
+import org.apache.xalan.extensions.ExtensionNamespacesManager;
+import org.apache.xalan.processor.XSLTSchema;
+import org.apache.xalan.res.XSLMessages;
+import org.apache.xalan.res.XSLTErrorResources;
+
+import org.apache.xalan.transformer.TransformerImpl;
+import org.apache.xml.dtm.DTM;
+import org.apache.xml.dtm.ref.ExpandedNameTable;
+import org.apache.xml.utils.IntStack;
+import org.apache.xml.utils.QName;
+import org.apache.xpath.XPath;
+import org.apache.xpath.XPathContext;
 
 /**
- * <meta name="usage" content="general"/>
  * This class represents the root object of the stylesheet tree.
+ * @xsl.usage general
  */
 public class StylesheetRoot extends StylesheetComposed
         implements java.io.Serializable, Templates
 {
+    static final long serialVersionUID = 3875353123529147855L;
+    
+    /**
+     * The flag for the setting of the optimize feature;
+     */    
+    private boolean m_optimizer = true;
+
+    /**
+     * The flag for the setting of the incremental feature;
+     */    
+    private boolean m_incremental = false;
+
+    /**
+     * The flag for the setting of the source_location feature;
+     */  
+    private boolean m_source_location = false;
+
+    /**
+     * State of the secure processing feature.
+     */
+    private boolean m_isSecureProcessing = false;
 
   /**
    * Uses an XSL stylesheet document.
@@ -114,7 +94,7 @@ public class StylesheetRoot extends StylesheetComposed
     }
     catch (TransformerException se)
     {
-      throw new TransformerConfigurationException("Can't init default templates!", se);
+      throw new TransformerConfigurationException(XSLMessages.createMessage(XSLTErrorResources.ER_CANNOT_INIT_DEFAULT_TEMPLATES, null), se); //"Can't init default templates!", se);
     }
   }
 
@@ -122,8 +102,8 @@ public class StylesheetRoot extends StylesheetComposed
    * The schema used when creating this StylesheetRoot
    * @serial
    */
-  private Hashtable m_availElems;
-
+  private HashMap m_availElems;
+  
   /**
    * Creates a StylesheetRoot and retains a pointer to the schema used to create this
    * StylesheetRoot.  The schema may be needed later for an element-available() function call.
@@ -136,7 +116,6 @@ public class StylesheetRoot extends StylesheetComposed
 
     this(listener);
     m_availElems = schema.getElemsAvailable();
-
   }
 
   /**
@@ -149,16 +128,65 @@ public class StylesheetRoot extends StylesheetComposed
     return true;
   }
 
+  /**
+   * Set the state of the secure processing feature.
+   */
+  public void setSecureProcessing(boolean flag)
+  {
+    m_isSecureProcessing = flag;
+  }
+  
+  /**
+   * Return the state of the secure processing feature.
+   */
+  public boolean isSecureProcessing()
+  {
+    return m_isSecureProcessing;
+  }
+
   /**
    * Get the hashtable of available elements.
    *
    * @return table of available elements, keyed by qualified names, and with 
    * values of the same qualified names.
    */
-  public Hashtable getAvailableElements()
+  public HashMap getAvailableElements()
   {
     return m_availElems;
   }
+  
+  private transient ExtensionNamespacesManager m_extNsMgr = null;
+  
+  /**
+   * Only instantiate an ExtensionNamespacesManager if one is called for
+   * (i.e., if the stylesheet contains  extension functions and/or elements).
+   */
+  public ExtensionNamespacesManager getExtensionNamespacesManager()
+  {
+     if (m_extNsMgr == null)
+       m_extNsMgr = new ExtensionNamespacesManager();
+     return m_extNsMgr;
+  }
+  
+  /**
+   * Get the vector of extension namespaces. Used to provide
+   * the extensions table access to a list of extension
+   * namespaces encountered during composition of a stylesheet.
+   */
+  public Vector getExtensions()
+  {
+    return m_extNsMgr != null ? m_extNsMgr.getExtensions() : null;
+  }  
+
+/*
+  public void runtimeInit(TransformerImpl transformer) throws TransformerException
+  {
+    System.out.println("StylesheetRoot.runtimeInit()");
+      
+  //    try{throw new Exception("StylesheetRoot.runtimeInit()");} catch(Exception e){e.printStackTrace();}
+
+    }
+*/  
 
   //============== Templates Interface ================
 
@@ -209,6 +237,9 @@ public class StylesheetRoot extends StylesheetComposed
    */
   public void recompose() throws TransformerException
   {
+    // Now we make a Vector that is going to hold all of the recomposable elements
+
+      Vector recomposableElements = new Vector();
 
     // First, we build the global import tree.
 
@@ -217,7 +248,7 @@ public class StylesheetRoot extends StylesheetComposed
 
       Vector importList = new Vector();
 
-      addImports(this, true, importList);
+      addImports(this, true, importList);            
 
       // Now we create an array and reverse the order of the importList vector.
       // We built the importList vector backwards so that we could use addElement
@@ -227,16 +258,18 @@ public class StylesheetRoot extends StylesheetComposed
 
       m_globalImportList = new StylesheetComposed[importList.size()];
 
-      for (int i = importList.size() - 1, j= 0; i >= 0; i--)
-        m_globalImportList[j++] = (StylesheetComposed) importList.elementAt(i);
-    }
-
-    // Now we make a Vector that is going to hold all of the recomposable elements
-
-    Vector recomposableElements = new Vector();
-
+      for (int i =  0, j= importList.size() -1; i < importList.size(); i++)
+      {  
+        m_globalImportList[j] = (StylesheetComposed) importList.elementAt(i);
+        // Build the global include list for this stylesheet.
+        // This needs to be done ahead of the recomposeImports
+        // because we need the info from the composed includes. 
+        m_globalImportList[j].recomposeIncludes(m_globalImportList[j]);
+        // Calculate the number of this import.    
+        m_globalImportList[j--].recomposeImports();        
+      }
+    }    
     // Next, we walk the import tree and add all of the recomposable elements to the vector.
-
     int n = getGlobalImportCount();
 
     for (int i = 0; i < n; i++)
@@ -251,9 +284,11 @@ public class StylesheetRoot extends StylesheetComposed
 
     // We set up the global variables that will hold the recomposed information.
 
-    m_outputProperties = new OutputProperties(Method.XML);
+
+    m_outputProperties = new OutputProperties(org.apache.xml.serializer.Method.UNKNOWN);
+//  m_outputProperties = new OutputProperties(Method.XML);
     
-    m_attrSets = new Hashtable();
+    m_attrSets = new HashMap();
     m_decimalFormatSymbols = new Hashtable();
     m_keyDecls = new Vector();
     m_namespaceAliasComposed = new Hashtable();
@@ -267,15 +302,33 @@ public class StylesheetRoot extends StylesheetComposed
     for (int i = recomposableElements.size() - 1; i >= 0; i--)
       ((ElemTemplateElement) recomposableElements.elementAt(i)).recompose(this);
     
+/*
+ * Backing out REE again, as it seems to cause some new failures
+ * which need to be investigated. -is
+ */      
+    // This has to be done before the initialization of the compose state, because 
+    // eleminateRedundentGlobals will add variables to the m_variables vector, which 
+    // it then copied in the ComposeState constructor.
+    
+//    if(true && org.apache.xalan.processor.TransformerFactoryImpl.m_optimize)
+//    {
+//          RedundentExprEliminator ree = new RedundentExprEliminator();
+//          callVisitors(ree);
+//          ree.eleminateRedundentGlobals(this);
+//    }
+          
+    initComposeState();
+
     // Need final composition of TemplateList.  This adds the wild cards onto the chains.
-    m_templateList.compose();
+    m_templateList.compose(this);
     
     // Need to clear check for properties at the same import level.
-    m_outputProperties.compose();
-
+    m_outputProperties.compose(this);
+    m_outputProperties.endCompose(this);
+    
     // Now call the compose() method on every element to give it a chance to adjust
     // based on composed values.
-
+    
     n = getGlobalImportCount();
 
     for (int i = 0; i < n; i++)
@@ -288,6 +341,11 @@ public class StylesheetRoot extends StylesheetComposed
         composeTemplates(included);
       }
     }
+    // Attempt to register any remaining unregistered extension namespaces.
+    if (m_extNsMgr != null)
+      m_extNsMgr.registerUnregisteredNamespaces();
+
+    clearComposeState();
   }
 
   /**
@@ -297,16 +355,18 @@ public class StylesheetRoot extends StylesheetComposed
    * the composed method called on it, and will have it's children's composed 
    * methods called.
    */
-  static void composeTemplates(ElemTemplateElement templ)
+  void composeTemplates(ElemTemplateElement templ) throws TransformerException
   {
 
-    templ.compose();
+    templ.compose(this);
 
     for (ElemTemplateElement child = templ.getFirstChildElem();
             child != null; child = child.getNextSiblingElem())
     {
       composeTemplates(child);
     }
+    
+    templ.endCompose(this);
   }
 
   /**
@@ -466,10 +526,10 @@ public class StylesheetRoot extends StylesheetComposed
   private boolean m_outputMethodSet = false;
 
   /**
-   * <meta name="usage" content="internal"/>
    * Find out if an output method has been set by the user.
    *
    * @return Value indicating whether an output method has been set by the user
+   * @xsl.usage internal
    */
   public boolean isOutputMethodSet()
   {
@@ -481,7 +541,7 @@ public class StylesheetRoot extends StylesheetComposed
    * Each entry is a vector of ElemAttributeSet objects.
    * @serial
    */
-  private Hashtable m_attrSets;
+  private HashMap m_attrSets;
 
   /**
    * Recompose the attribute-set declarations.
@@ -490,16 +550,16 @@ public class StylesheetRoot extends StylesheetComposed
    */
   void recomposeAttributeSets(ElemAttributeSet attrSet)
   {
-    Vector attrSetList = (Vector) m_attrSets.get(attrSet.getName());
+    ArrayList attrSetList = (ArrayList) m_attrSets.get(attrSet.getName());
 
     if (null == attrSetList)
     {
-      attrSetList = new Vector();
+      attrSetList = new ArrayList();
 
       m_attrSets.put(attrSet.getName(), attrSetList);
     }
 
-    attrSetList.addElement(attrSet);
+    attrSetList.add(attrSet);
   }
 
   /**
@@ -512,10 +572,10 @@ public class StylesheetRoot extends StylesheetComposed
    *
    * @throws ArrayIndexOutOfBoundsException
    */
-  public Vector getAttributeSetComposed(QName name)
+  public ArrayList getAttributeSetComposed(QName name)
           throws ArrayIndexOutOfBoundsException
   {
-    return (Vector) m_attrSets.get(name);
+    return (ArrayList) m_attrSets.get(name);
   }
 
   /**
@@ -687,9 +747,6 @@ public class StylesheetRoot extends StylesheetComposed
    * @param xctxt non-null reference to XPath runtime execution context.
    * @param targetNode non-null reference of node that the template must match.
    * @param mode qualified name of the node, or null.
-   * @param maxImportLevel The maximum importCountComposed that we should consider or -1
-   *        if we should consider all import levels.  This is used by apply-imports to
-   *        access templates that have been overridden.
    * @param quietConflictWarnings true if conflict warnings should not be reported.
    *
    * @return reference to ElemTemplate that is the best match for targetNode, or 
@@ -698,14 +755,48 @@ public class StylesheetRoot extends StylesheetComposed
    * @throws TransformerException
    */
   public ElemTemplate getTemplateComposed(XPathContext xctxt,
-                                          Node targetNode,
+                                          int targetNode,
                                           QName mode,
-                                          int maxImportLevel,
-                                          boolean quietConflictWarnings)
+                                          boolean quietConflictWarnings,
+                                          DTM dtm)
             throws TransformerException
   {
-    return m_templateList.getTemplate(xctxt, targetNode, mode, maxImportLevel,
-                                                                quietConflictWarnings);
+    return m_templateList.getTemplate(xctxt, targetNode, mode, 
+                                      quietConflictWarnings,
+                                      dtm);
+  }
+  
+  /**
+   * Get an "xsl:template" property by node match. This looks in the imports as
+   * well as this stylesheet.
+   * @see <a href="http://www.w3.org/TR/xslt#section-Defining-Template-Rules">section-Defining-Template-Rules in XSLT Specification</a>
+   *
+   * @param xctxt non-null reference to XPath runtime execution context.
+   * @param targetNode non-null reference of node that the template must match.
+   * @param mode qualified name of the node, or null.
+   * @param maxImportLevel The maximum importCountComposed that we should consider or -1
+   *        if we should consider all import levels.  This is used by apply-imports to
+   *        access templates that have been overridden.
+   * @param endImportLevel The count of composed imports
+   * @param quietConflictWarnings true if conflict warnings should not be reported.
+   *
+   * @return reference to ElemTemplate that is the best match for targetNode, or 
+   *         null if no match could be made.
+   *
+   * @throws TransformerException
+   */
+  public ElemTemplate getTemplateComposed(XPathContext xctxt,
+                                          int targetNode,
+                                          QName mode,
+                                          int maxImportLevel, int endImportLevel,
+                                          boolean quietConflictWarnings,
+                                          DTM dtm)
+            throws TransformerException
+  {
+    return m_templateList.getTemplate(xctxt, targetNode, mode, 
+                                      maxImportLevel, endImportLevel,
+                                      quietConflictWarnings,
+                                      dtm);
   }
 
   /**
@@ -737,7 +828,11 @@ public class StylesheetRoot extends StylesheetComposed
   {
     // Don't overide higher priority variable        
     if (getVariableOrParamComposed(elemVar.getName()) == null)
+    {
+      elemVar.setIsTopLevel(true);        // Mark as a top-level variable or param
+      elemVar.setIndex(m_variables.size());
       m_variables.addElement(elemVar);
+    }
   }
 
   /**
@@ -820,12 +915,12 @@ public class StylesheetRoot extends StylesheetComposed
    * @throws TransformerException
    */
   public WhiteSpaceInfo getWhiteSpaceInfo(
-          XPathContext support, Element targetElement) throws TransformerException
+          XPathContext support, int targetElement, DTM dtm) throws TransformerException
   {
 
     if (null != m_whiteSpaceInfoList)
       return (WhiteSpaceInfo) m_whiteSpaceInfoList.getTemplate(support,
-              targetElement, null, -1, false);
+              targetElement, null, false, dtm);
     else
       return null;
   }
@@ -842,22 +937,23 @@ public class StylesheetRoot extends StylesheetComposed
    * @throws TransformerException
    */
   public boolean shouldStripWhiteSpace(
-          XPathContext support, Element targetElement) throws TransformerException
+          XPathContext support, int targetElement) throws TransformerException
   {
     if (null != m_whiteSpaceInfoList)
     {
-      while(null != targetElement)
+      while(DTM.NULL != targetElement)
       {
+        DTM dtm = support.getDTM(targetElement);
         WhiteSpaceInfo info = (WhiteSpaceInfo) m_whiteSpaceInfoList.getTemplate(support,
-                targetElement, null, -1, false);
+                targetElement, null, false, dtm);
         if(null != info)
           return info.getShouldStripSpace();
-          
-        Node parent = targetElement.getParentNode();
-        if(null != parent && Node.ELEMENT_NODE == parent.getNodeType())
-          targetElement = (Element)parent;
+        
+        int parent = dtm.getParent(targetElement);
+        if(DTM.NULL != parent && DTM.ELEMENT_NODE == dtm.getNodeType(parent))
+          targetElement = parent;
         else
-          targetElement = null;
+          targetElement = DTM.NULL;
       }
     }
     return false;
@@ -877,18 +973,18 @@ public class StylesheetRoot extends StylesheetComposed
 
 
   /**
-   * <meta name="usage" content="advanced"/>
    * The default template to use for text nodes if we don't find
    * anything else.  This is initialized in initDefaultRule().
    * @serial
+   * @xsl.usage advanced
    */
   private ElemTemplate m_defaultTextRule;
 
   /**
-   * <meta name="usage" content="advanced"/>
    * Get the default template for text.
    *
    * @return the default template for text.
+   * @xsl.usage advanced
    */
   public final ElemTemplate getDefaultTextRule()
   {
@@ -896,18 +992,18 @@ public class StylesheetRoot extends StylesheetComposed
   }
 
   /**
-   * <meta name="usage" content="advanced"/>
    * The default template to use if we don't find anything
    * else.  This is initialized in initDefaultRule().
    * @serial
+   * @xsl.usage advanced
    */
   private ElemTemplate m_defaultRule;
 
   /**
-   * <meta name="usage" content="advanced"/>
    * Get the default template for elements.
    *
    * @return the default template for elements.
+   * @xsl.usage advanced
    */
   public final ElemTemplate getDefaultRule()
   {
@@ -915,7 +1011,6 @@ public class StylesheetRoot extends StylesheetComposed
   }
 
   /**
-   * <meta name="usage" content="advanced"/>
    * The default template to use for the root if we don't find
    * anything else.  This is initialized in initDefaultRule().
    * We kind of need this because the defaultRule isn't good
@@ -924,19 +1019,39 @@ public class StylesheetRoot extends StylesheetComposed
    * Don't know if this is really a good idea or not.
    * I suspect it is not.
    * @serial
+   * @xsl.usage advanced
    */
   private ElemTemplate m_defaultRootRule;
 
   /**
-   * <meta name="usage" content="advanced"/>
    * Get the default template for a root node.
    *
    * @return The default template for a root node.
+   * @xsl.usage advanced
    */
   public final ElemTemplate getDefaultRootRule()
   {
     return m_defaultRootRule;
   }
+  
+  /**
+   * The start rule to kick off the transformation.
+   * @serial
+   * @xsl.usage advanced
+   */
+  private ElemTemplate m_startRule;
+
+  /**
+   * Get the default template for a root node.
+   *
+   * @return The default template for a root node.
+   * @xsl.usage advanced
+   */
+  public final ElemTemplate getStartRule()
+  {
+    return m_startRule;
+  }
+
 
   /**
    * Used for default selection.
@@ -964,7 +1079,10 @@ public class StylesheetRoot extends StylesheetComposed
     ElemApplyTemplates childrenElement = new ElemApplyTemplates();
 
     childrenElement.setIsDefaultTemplate(true);
+    childrenElement.setSelect(m_selectDefault);
     m_defaultRule.appendChild(childrenElement);
+    
+    m_startRule = m_defaultRule;
 
     // -----------------------------
     m_defaultTextRule = new ElemTemplate();
@@ -996,6 +1114,7 @@ public class StylesheetRoot extends StylesheetComposed
 
     childrenElement.setIsDefaultTemplate(true);
     m_defaultRootRule.appendChild(childrenElement);
+    childrenElement.setSelect(m_selectDefault);
   }
 
   /**
@@ -1071,4 +1190,216 @@ public class StylesheetRoot extends StylesheetComposed
         }
       }
     } // end QuickSort2  */
+    
+    private transient ComposeState m_composeState;
+    
+    /**
+     * Initialize a new ComposeState.
+     */
+    void initComposeState()
+    {
+      m_composeState = new ComposeState();
+    }
+
+    /**
+     * Return class to track state global state during the compose() operation.
+     * @return ComposeState reference, or null if endCompose has been called.
+     */
+    ComposeState getComposeState()
+    {
+      return m_composeState;
+    }
+    
+    /**
+     * Clear the compose state.
+     */
+    private void clearComposeState()
+    {
+      m_composeState = null;
+    }
+
+    private String m_extensionHandlerClass = 
+        "org.apache.xalan.extensions.ExtensionHandlerExsltFunction";
+    
+    /**
+     * This internal method allows the setting of the java class
+     * to handle the extension function (if other than the default one).
+     * 
+     * @xsl.usage internal
+     */
+    public String setExtensionHandlerClass(String handlerClassName) {
+        String oldvalue = m_extensionHandlerClass;
+        m_extensionHandlerClass = handlerClassName;
+        return oldvalue;
+    } 
+    /**
+     * 
+     * @xsl.usage internal
+     */
+    public String getExtensionHandlerClass() {
+        return m_extensionHandlerClass;
+    }
+        
+    /**
+     * Class to track state global state during the compose() operation.
+     */
+    class ComposeState
+    {
+      ComposeState()
+      {
+        int size = m_variables.size();
+        for (int i = 0; i < size; i++) 
+        {
+          ElemVariable ev = (ElemVariable)m_variables.elementAt(i);
+          m_variableNames.addElement(ev.getName());
+        }
+        
+      }
+      
+      private ExpandedNameTable m_ent = new ExpandedNameTable();
+      
+      /**
+       * Given a qualified name, return an integer ID that can be 
+       * quickly compared.
+       *
+       * @param qname a qualified name object, must not be null.
+       *
+       * @return the expanded-name id of the qualified name.
+       */
+      public int getQNameID(QName qname)
+      {
+        
+        return m_ent.getExpandedTypeID(qname.getNamespace(), 
+                                       qname.getLocalName(),
+                                       // The type doesn't matter for our 
+                                       // purposes. 
+                                       org.apache.xml.dtm.DTM.ELEMENT_NODE);
+      }
+      
+      /**
+       * A Vector of the current params and QNames within the current template.
+       * Set by ElemTemplate and used by ProcessorVariable.
+       */
+      private java.util.Vector m_variableNames = new java.util.Vector();
+            
+      /**
+       * Add the name of a qualified name within the template.  The position in 
+       * the vector is its ID.
+       * @param qname A qualified name of a param or variable, should be non-null.
+       * @return the index where the variable was added.
+       */
+      int addVariableName(final org.apache.xml.utils.QName qname)
+      {
+        int pos = m_variableNames.size();
+        m_variableNames.addElement(qname);
+        int frameSize = m_variableNames.size() - getGlobalsSize();
+        if(frameSize > m_maxStackFrameSize)
+          m_maxStackFrameSize++;
+        return pos;
+      }
+      
+      void resetStackFrameSize()
+      {
+        m_maxStackFrameSize = 0;
+      }
+      
+      int getFrameSize()
+      {
+        return m_maxStackFrameSize;
+      }
+      
+      /**
+       * Get the current size of the stack frame.  Use this to record the position 
+       * in a template element at startElement, so that it can be popped 
+       * at endElement.
+       */
+      int getCurrentStackFrameSize()
+      {
+        return m_variableNames.size();
+      }
+      
+      /**
+       * Set the current size of the stack frame.
+       */
+      void setCurrentStackFrameSize(int sz)
+      {
+        m_variableNames.setSize(sz);
+      }
+      
+      int getGlobalsSize()
+      {
+        return m_variables.size();
+      }
+      
+      IntStack m_marks = new IntStack();
+      
+      void pushStackMark()
+      {
+        m_marks.push(getCurrentStackFrameSize());
+      }
+      
+      void popStackMark()
+      {
+        int mark = m_marks.pop();
+        setCurrentStackFrameSize(mark);
+      }
+      
+      /**
+       * Get the Vector of the current params and QNames to be collected 
+       * within the current template.
+       * @return A reference to the vector of variable names.  The reference 
+       * returned is owned by this class, and so should not really be mutated, or 
+       * stored anywhere.
+       */
+      java.util.Vector getVariableNames()
+      {
+        return m_variableNames;
+      }
+      
+      private int m_maxStackFrameSize;
+
+    }
+    
+    /**
+     * @return Optimization flag
+     */
+    public boolean getOptimizer() {
+        return m_optimizer;
+    }
+
+    /**
+     * @param b Optimization flag
+     */
+    public void setOptimizer(boolean b) {
+        m_optimizer = b;
+    }
+
+    /**
+     * @return Incremental flag
+     */
+    public boolean getIncremental() {
+        return m_incremental;
+    }
+
+    /**
+     * @return source location flag
+     */
+    public boolean getSource_location() {
+        return m_source_location;
+    }
+
+    /**
+     * @param b Incremental flag
+     */
+    public void setIncremental(boolean b) {
+        m_incremental = b;
+    }
+
+    /**
+     * @param b Source location flag
+     */
+    public void setSource_location(boolean b) {
+        m_source_location = b;
+    }
+
 }
