@@ -1,77 +1,47 @@
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the  "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Copyright (c) 1999 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xalan" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, Lotus
- * Development Corporation., http://www.lotus.com.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+/*
+ * $Id: DOMBuilder.java 472634 2006-11-08 20:43:55Z jycli $
  */
 package org.apache.xml.utils;
 
-import org.apache.xalan.res.XSLMessages;
-import org.apache.xpath.res.XPATHErrorResources;
-import org.apache.xml.utils.NodeVector;
+import java.util.Stack;
+import java.util.Vector;
 
-import org.xml.sax.ext.LexicalHandler;
+import org.apache.xml.res.XMLErrorResources;
+import org.apache.xml.res.XMLMessages;
+
+import org.w3c.dom.Document;
+import org.w3c.dom.DocumentFragment;
+import org.w3c.dom.Element;
+import org.w3c.dom.Node;
+import org.w3c.dom.Text;
+import org.w3c.dom.CDATASection;
+
+import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.Locator;
-import org.xml.sax.Attributes;
-
-import org.w3c.dom.*;  // we pretty much use everything in the DOM here.
-
+import org.xml.sax.ext.LexicalHandler;
 /**
- * <meta name="usage" content="general"/>
  * This class takes SAX events (in addition to some extra events
  * that SAX doesn't handle yet) and adds the result to a document
  * or document fragment.
+ * @xsl.usage general
  */
 public class DOMBuilder
         implements ContentHandler, LexicalHandler
@@ -82,13 +52,22 @@ public class DOMBuilder
 
   /** Current node           */
   protected Node m_currentNode = null;
+  
+  /** The root node          */
+  protected Node m_root = null;
+  
+  /** The next sibling node  */
+  protected Node m_nextSibling = null;
 
   /** First node of document fragment or null if not a DocumentFragment     */
   public DocumentFragment m_docFrag = null;
 
   /** Vector of element nodes          */
-  protected NodeVector m_elemStack = new NodeVector();
-
+  protected Stack m_elemStack = new Stack();
+  
+  /** Namespace support */
+  protected Vector m_prefixMappings = new Vector();
+  
   /**
    * DOMBuilder instance constructor... it will add the DOM nodes
    * to the document fragment.
@@ -99,7 +78,10 @@ public class DOMBuilder
   public DOMBuilder(Document doc, Node node)
   {
     m_doc = doc;
-    m_currentNode = node;
+    m_currentNode = m_root = node;
+    
+    if (node instanceof Element)
+      m_elemStack.push(node);
   }
 
   /**
@@ -127,16 +109,23 @@ public class DOMBuilder
   }
 
   /**
-   * Get the root node of the DOM being created.  This
-   * is either a Document or a DocumentFragment.
+   * Get the root document or DocumentFragment of the DOM being created.
    *
    * @return The root document or document fragment if not null
    */
-  public Node getRootNode()
+  public Node getRootDocument()
   {
     return (null != m_docFrag) ? (Node) m_docFrag : (Node) m_doc;
   }
-
+  
+  /**
+   * Get the root node of the DOM tree.
+   */
+  public Node getRootNode()
+  {
+    return m_root;
+  }
+  
   /**
    * Get the node currently being processed.
    *
@@ -146,6 +135,27 @@ public class DOMBuilder
   {
     return m_currentNode;
   }
+  
+  /**
+   * Set the next sibling node, which is where the result nodes 
+   * should be inserted before.
+   * 
+   * @param nextSibling the next sibling node.
+   */
+  public void setNextSibling(Node nextSibling)
+  {
+    m_nextSibling = nextSibling;
+  }
+  
+  /**
+   * Return the next sibling node.
+   * 
+   * @return the next sibling node.
+   */
+  public Node getNextSibling()
+  {
+    return m_nextSibling;
+  }
 
   /**
    * Return null since there is no Writer for this class.
@@ -169,13 +179,19 @@ public class DOMBuilder
 
     if (null != currentNode)
     {
-      currentNode.appendChild(newNode);
+      if (currentNode == m_root && m_nextSibling != null)
+        currentNode.insertBefore(newNode, m_nextSibling);
+      else
+        currentNode.appendChild(newNode);
 
       // System.out.println(newNode.getNodeName());
     }
     else if (null != m_docFrag)
     {
-      m_docFrag.appendChild(newNode);
+      if (m_nextSibling != null)
+        m_docFrag.insertBefore(newNode, m_nextSibling);
+      else
+        m_docFrag.appendChild(newNode);
     }
     else
     {
@@ -189,8 +205,8 @@ public class DOMBuilder
         if ((null != data) && (data.trim().length() > 0))
         {
           throw new org.xml.sax.SAXException(
-            XSLMessages.createXPATHMessage(
-              XPATHErrorResources.ER_CANT_OUTPUT_TEXT_BEFORE_DOC, null));  //"Warning: can't output text before document element!  Ignoring...");
+            XMLMessages.createXMLMessage(
+              XMLErrorResources.ER_CANT_OUTPUT_TEXT_BEFORE_DOC, null));  //"Warning: can't output text before document element!  Ignoring...");
         }
 
         ok = false;
@@ -199,14 +215,21 @@ public class DOMBuilder
       {
         if (m_doc.getDocumentElement() != null)
         {
+          ok = false;
+          
           throw new org.xml.sax.SAXException(
-            XSLMessages.createXPATHMessage(
-              XPATHErrorResources.ER_CANT_HAVE_MORE_THAN_ONE_ROOT, null));  //"Can't have more than one root on a DOM!");
+            XMLMessages.createXMLMessage(
+              XMLErrorResources.ER_CANT_HAVE_MORE_THAN_ONE_ROOT, null));  //"Can't have more than one root on a DOM!");
         }
       }
 
       if (ok)
-        m_doc.appendChild(newNode);
+      {
+        if (m_nextSibling != null)
+          m_doc.insertBefore(newNode, m_nextSibling);
+        else
+          m_doc.appendChild(newNode);
+      }
     }
   }
 
@@ -286,7 +309,7 @@ public class DOMBuilder
    *
    *
    * @param ns The namespace of the node
-   * @param localName The local part of the qualified name 
+   * @param localName The local part of the qualified name
    * @param name The element name.
    * @param atts The attributes attached to the element, if any.
    * @see #endElement
@@ -299,8 +322,10 @@ public class DOMBuilder
 
     Element elem;
 
+	// Note that the namespace-aware call must be used to correctly
+	// construct a Level 2 DOM, even for non-namespaced nodes.
     if ((null == ns) || (ns.length() == 0))
-      elem = m_doc.createElement(name);
+      elem = m_doc.createElementNS(null,name);
     else
       elem = m_doc.createElementNS(ns, name);
 
@@ -309,43 +334,65 @@ public class DOMBuilder
     try
     {
       int nAtts = atts.getLength();
-  
+
       if (0 != nAtts)
       {
         for (int i = 0; i < nAtts; i++)
         {
-  
+
           //System.out.println("type " + atts.getType(i) + " name " + atts.getLocalName(i) );
           // First handle a possible ID attribute
           if (atts.getType(i).equalsIgnoreCase("ID"))
             setIDAttribute(atts.getValue(i), elem);
-  
+
           String attrNS = atts.getURI(i);
-          
-          if(attrNS == null)
-            attrNS = ""; // defensive, shouldn't have to do this.
-  
+
+          if("".equals(attrNS))
+            attrNS = null; // DOM represents no-namespace as null
+
           // System.out.println("attrNS: "+attrNS+", localName: "+atts.getQName(i)
           //                   +", qname: "+atts.getQName(i)+", value: "+atts.getValue(i));
           // Crimson won't let us set an xmlns: attribute on the DOM.
           String attrQName = atts.getQName(i);
-          if ((attrNS.length() == 0) /* || attrQName.startsWith("xmlns:") || attrQName.equals("xmlns") */)
-            elem.setAttribute(attrQName, atts.getValue(i));
-          else
-          {
-  
-            // elem.setAttributeNS(atts.getURI(i), atts.getLocalName(i), atts.getValue(i));
-            elem.setAttributeNS(attrNS, attrQName, atts.getValue(i));
+
+          // In SAX, xmlns[:] attributes have an empty namespace, while in DOM they 
+          // should have the xmlns namespace
+          if (attrQName.startsWith("xmlns:") || attrQName.equals("xmlns")) {
+            attrNS = "http://www.w3.org/2000/xmlns/";
           }
+
+          // ALWAYS use the DOM Level 2 call!
+          elem.setAttributeNS(attrNS,attrQName, atts.getValue(i));
         }
       }
       
-      // append(elem);
-  
-      m_elemStack.push(elem);
-  
-      m_currentNode = elem;
+      /*
+       * Adding namespace nodes to the DOM tree;
+       */
+      int nDecls = m_prefixMappings.size();
       
+      String prefix, declURL;
+      
+      for (int i = 0; i < nDecls; i += 2)
+      {
+        prefix = (String) m_prefixMappings.elementAt(i);
+
+        if (prefix == null)
+          continue;
+
+        declURL = (String) m_prefixMappings.elementAt(i + 1);
+
+        elem.setAttributeNS("http://www.w3.org/2000/xmlns/", prefix, declURL);
+      }
+      
+      m_prefixMappings.clear();
+    
+      // append(elem);
+
+      m_elemStack.push(elem);
+
+      m_currentNode = elem;
+
       // append(elem);
     }
     catch(java.lang.Exception de)
@@ -353,10 +400,13 @@ public class DOMBuilder
       // de.printStackTrace();
       throw new org.xml.sax.SAXException(de);
     }
-    
+
   }
 
   /**
+
+
+
    * Receive notification of the end of an element.
    *
    * <p>The SAX parser will invoke this method at the end of every
@@ -375,7 +425,8 @@ public class DOMBuilder
   public void endElement(String ns, String localName, String name)
           throws org.xml.sax.SAXException
   {
-    m_currentNode = m_elemStack.popAndTop();
+    m_elemStack.pop();
+    m_currentNode = m_elemStack.isEmpty() ? null : (Node)m_elemStack.peek();
   }
 
   /**
@@ -427,9 +478,15 @@ public class DOMBuilder
     }
 
     String s = new String(ch, start, length);
-    Text text = m_doc.createTextNode(s);
-
-    append(text);
+    Node childNode;
+    childNode =  m_currentNode != null ? m_currentNode.getLastChild(): null;
+    if( childNode != null && childNode.getNodeType() == Node.TEXT_NODE ){
+       ((Text)childNode).appendData(s);
+    }
+    else{
+       Text text = m_doc.createTextNode(s);
+       append(text);
+    }
   }
 
   /**
@@ -440,12 +497,12 @@ public class DOMBuilder
    *
    * @param ch Array containing the characters
    * @param start Index to start of characters in the array
-   * @param length Number of characters in the array 
+   * @param length Number of characters in the array
    */
   public void charactersRaw(char ch[], int start, int length)
           throws org.xml.sax.SAXException
   {
-    if(isOutsideDocElem()  
+    if(isOutsideDocElem()
        && org.apache.xml.utils.XMLCharacterRecognizer.isWhiteSpace(ch, start, length))
       return;  // avoid DOM006 Hierarchy request error
 
@@ -528,10 +585,10 @@ public class DOMBuilder
 
     append(m_doc.createTextNode(s));
   }
-  
+
   /**
    * Tell if the current node is outside the document element.
-   * 
+   *
    * @return true if the current node is outside the document element.
    */
    private boolean isOutsideDocElem()
@@ -587,6 +644,7 @@ public class DOMBuilder
   public void startCDATA() throws org.xml.sax.SAXException
   {
     m_inCData = true;
+    append(m_doc.createCDATASection(""));
   }
 
   /**
@@ -624,13 +682,14 @@ public class DOMBuilder
    */
   public void cdata(char ch[], int start, int length) throws org.xml.sax.SAXException
   {
-    if(isOutsideDocElem()  
+    if(isOutsideDocElem()
        && org.apache.xml.utils.XMLCharacterRecognizer.isWhiteSpace(ch, start, length))
       return;  // avoid DOM006 Hierarchy request error
 
     String s = new String(ch, start, length);
 
-    append(m_doc.createCDATASection(s));
+    CDATASection section  =(CDATASection) m_currentNode.getLastChild();
+    section.appendData(s);
   }
 
   /**
@@ -696,29 +755,11 @@ public class DOMBuilder
   public void startPrefixMapping(String prefix, String uri)
           throws org.xml.sax.SAXException
   {
-
-    /*
-    // Not sure if this is needed or wanted
-    // Also, it fails in the stree.
-    if((null != m_currentNode)
-       && (m_currentNode.getNodeType() == Node.ELEMENT_NODE))
-    {
-      String qname;
-      if(((null != prefix) && (prefix.length() == 0))
-         || (null == prefix))
-        qname = "xmlns";
-      else
-        qname = "xmlns:"+prefix;
-
-      Element elem = (Element)m_currentNode;
-      String val = elem.getAttribute(qname);
-      if(val == null)
-      {
-        elem.setAttributeNS("http://www.w3.org/XML/1998/namespace",
-                            qname, uri);
-      }
-    }
-    */
+	      if(null == prefix || prefix.equals(""))
+	        prefix = "xmlns";
+	      else prefix = "xmlns:"+prefix;
+	      m_prefixMappings.addElement(prefix);
+	      m_prefixMappings.addElement(uri); 
   }
 
   /**
